// jc.java
/*
 * The standalone interpreter for jC programs
 * jbgarcia@uvigo.es
 * 201306
 */

package jc;

import javax.swing.*;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;

/**
 * Standalone jc interpreter
 * @author baltasarq
 */

public class Main {
    public static final String ExecutableExt = "jcb";
    public static final String ExitCodeVar = "_exitCode";

    public static final String Name = "jC";
    public static final String Version = "v0.3 201306";
    public static int ExitCode = 0;

    public static void execute(File project)
    {
        final bsh.Interpreter terp = new bsh.Interpreter();
        final JFrame consoleWindow = new JFrame();
        final bsh.util.JConsole console = new bsh.util.JConsole();
        final JLabel lblStatus = new JLabel();
        ExitCode = 0;

        try {
            String programName = project.getName();

            // Get the interpreter file name
            project = project.getCanonicalFile();
            String fileName = project.getAbsolutePath();
            int dotPos = fileName.lastIndexOf( '.' );
            if ( dotPos > 0 ) {
                fileName = fileName.substring( 0, dotPos );
            }
            fileName += '.';
            fileName += ExecutableExt;

            // Create the console window
            consoleWindow.addWindowListener(new WindowAdapter() {
                @Override
                public void windowClosing(WindowEvent evt) {
                     System.exit( ExitCode );
                }
            });
            consoleWindow.setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
            consoleWindow.setMinimumSize( new Dimension( 500, 400) );
            consoleWindow.setSize( consoleWindow.getMinimumSize() );
            consoleWindow.setTitle( Name + " - [ " + programName + " ]" );
            consoleWindow.setLayout( new BorderLayout() );

            // Create the console itself
            consoleWindow.add( console, BorderLayout.CENTER );
            consoleWindow.add( lblStatus, BorderLayout.SOUTH );
            consoleWindow.pack();
            consoleWindow.setVisible( true );
            console.setVisible( true );
            System.setOut( console.getOut() );
            System.setIn( console.getInputStream() );
            System.setErr( console.getErr() );

            terp.setConsole( console );

            if ( !new File( fileName ).exists() ) {
                throw new RuntimeException( "Program \n\t'" + fileName + "'\nhas not been compiled yet." );
            }

            // Config. the interpreter and execute the code
            terp.setExitOnEOF( true );
            terp.source( fileName );

            ExitCode = ( (Integer) terp.get( ExitCodeVar ) ).intValue();

        } catch (FileNotFoundException ex) {
            ExitCode = 1;
            System.err.println( "\n**Missing file: " + ex.getMessage() );
        } catch (IOException ex) {
            ExitCode = 1;
            System.err.println( "\n**Unexpected I/O exception" );
        } catch (bsh.EvalError ex) {
            ExitCode = 1;
            String error = "unknown";

            // Report error
            if ( ex != null
              && ex.getMessage() != null )
            {
                error = ex.getErrorText();
            }

            System.err.println( "\n***Error : " + error );
        } catch(RuntimeException exc) {
            ExitCode = 1;
            System.err.println( "\n****Error: " + exc.getMessage() );
        } catch(Exception exc) {
            ExitCode = 1;
            System.err.println( "\nUnexpected error: " + exc.getMessage() );
        }

        // End
        lblStatus.setText( "Exit code: " + Integer.toString( ExitCode ) );
        return;
    }

    public static void showVersion()
    {
        System.out.println( Name + " " + Version + "\n");
    }

    public static void showHelp()
    {
        System.out.println(
                "\n\tjc <filename[.jcb]> - Executes the jC program in filename.\n\n"
        );
    }

    public static String[] chkArguments(String[] args)
    {
        ArrayList<String> v = new ArrayList<String>();

        for(String arg : args) {
            v.add( arg.trim() );
        }

        return ( v.toArray( new String[ v.size() ] ) );
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args)
    {
        args = chkArguments( args );

        if ( args.length != 1 ) {
            showVersion();
            showHelp();
        } else {
            execute( new File( args[ 0 ] ) );
        }
    }

}
