
import javax.swing.*;
import java.awt.*;
import java.awt.PointerInfo;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.image.BufferedImage;
import java.lang.Math;

/**
    jC gw media library package
    @name   gw (jC media Library)
    @brief  Common graphic functions
    @date   2010-12-1
    @author jbgarcia@uvigo.es
*/
public class gw {

public static final int MinWidth = 100;
public static final int MinHeight = 100;

/*
    Colores:
    0 - Black, Negro
    1 - Blue, Azul
    2 - Cyan, Cian
    3 - Gray, Grey, Gris
    4 - Green, Verde
    5 - Magenta
    6 - Orange, Naranja
    7 - Pink, Rosa
    8 - Red, Rojo
    9 - White, Blanco
    10 - Yellow, Amarillo
    11 - Brown, marrón
    12 - Dark gray, gris oscuro
    13 - Light Gray, gris claro
*/

private static final Color[] Colors = {
    Color.black, Color.blue,
    Color.cyan, Color.gray,
    Color.green, Color.magenta,
    Color.orange, Color.pink,
    Color.red, Color.white,
    Color.yellow, new Color(165, 42, 42 ),
    Color.darkGray, Color.lightGray
};

public static final int MaxColors = Colors.length;

public static final int Black = 0;
public static final int Negro = Black;

public static final int Blue = 1;
public static final int Azul = Blue;

public static final int Cyan = 2;
public static final int Cian = Cyan;

public static final int Gray = 3;
public static final int Gris = Gray;
public static final int Grey = Gray;

public static final int Green = 4;
public static final int Verde = Green;

public static final int Magenta = 5;

public static final int Orange = 6;
public static final int Naranja = Orange;

public static final int Pink = 7;
public static final int Rosa = Pink;

public static final int Red = 8;
public static final int Rojo = Red;

public static final int White = 9;
public static final int Blanco = White;

public static final int Yellow = 10;
public static final int Amarillo = Yellow;

public static final int Brown = 11;
public static final int Marron = Brown;

public static final int DarkGray = 12;
public static final int DarkGrey = 12;
public static final int GrisOscuro = DarkGray;

public static final int LightGray = 13;
public static final int LightGrey = 13;
public static final int GrisClaro = LightGray;

/**
 * iJava sets the beginning of the coordinates at the lower left corner.
 * This means that the Y (vertical) coordinate must be decoded accordingly.
 * @param y The Y coordinate in the iJava graphics screen.
 * @return The decodified Y coordinate.
 */
private static int decodeY(int y)
{
    return Math.abs( getHeight() - y - 1 );
}

/**
 * iJava sets the beginning of the coordinates at the lower left corner.
 * This means that the Y (vertical) coordinate must be decoded accordingly.
 * @param ys A vector of Y coordinates in the iJava graphics screen.
 */
private static void decodeY(int[] ys)
{
    for(int i = 0; i < ys.length; ++i) {
        ys[ i ] = decodeY( ys[ i ] );
    }

    return;
}

/**
 * Converts a color from the above list, to a
 * Color object from the Java's AWT system.
 * @param c the color, as an int from black (0) to yellow(10)
 * @return the corresponding color, as a Color object.
 */
private static Color decodeColor(int c)
{
    if ( c < 0
      || c >= Colors.length )
    {
        c = 0;
    }

    return Colors[ c ];
}

/**
 * Convert a color from its r, g, b components
 * @param r the red component, as int
 * @param g the green component, as int
 * @param b the blue component, as int
 * @return the corresponding, as a Color object.
 */
private static Color decodeColor(int r, int g, int b)
{
	return new Color( r, g, b );
}

/**
 * Convert a color from its r, g, b components
 * @param r the red component, as int
 * @param g the green component, as int
 * @param b the blue component, as int
 * @return the corresponding, as an index of the list of colors above.
 */
private static int encodeColor(int r, int g, int b)
{
   return encodeColor( new Color( r, g, b ) );
}

/**
 * Convert a color to an index of the colors available
 * @param r the red component, as int
 * @param g the green component, as int
 * @param b the blue component, as int
 * @return the corresponding, as an index of the list of colors above.
 */
public static int encodeColor(Color c)
{
	int toret = 0;

    for(int i = 0; i < Colors.length; ++i) {
		if ( c.equals( Colors[ i ] ) ) {
			toret = i;
			break;
		}
	}

	return toret;
}

/**
 * Changes the color used for the foreground
 * @parameter color as an int, indexing the list of colors
 */
public static void setFgColor(int color)
{
    fg = decodeColor( color );
}

/**
 * Changes the color used for the foreground
 * @parameter color as an int, indexing the list of colors
 */
public static void setColor(int color)
{
    setFgColor( color );
}

/**
 * Changes the color used for the foreground
 * @param r the red component, as int
 * @param g the green component, as int
 * @param b the blue component, as int
 */
public static void setColor(int r, int g, int b)
{
	fg = decodeColor( r, g, b );
}

/**
 * Changes the colors used for foreground and background.
 * @param fg the new color for the foreground, as int
 * @param bg the new color for the background, as int
 */
public static void setColors(int fg, int bg)
{
    setBgColor( bg );
    setFgColor( fg );
}

/**
 * Changes the color used for the background
 * @parameter color as an int, indexing the list of colors
 */
public static void setBgColor(int bgc)
{
    final Graphics grfs;

    chkOpen();
	bg = decodeColor( bgc );
    grfs = canvas.getGraphics();
    grfs.setColor( bg );
    grfs.fillRect( 0, 0, getWidth(), getHeight() );
    window.repaint();
}

/**
 * Draws a line in the graphics window, given its
 * starting and ending point.
 * @param x1 the x coordinate of the starting point
 * @param y1 the y coordinate of the starting point
 * @param x2 the x coordinate of the ending point
 * @param y2 the y coordinate of the ending point
 */
public static void line(int x1, int y1, int x2, int y2)
{
    final Graphics grfs;

    chkOpen();
    grfs = canvas.getGraphics();
    grfs.setColor( fg );
    grfs.drawLine( x1, decodeY( y1 ), x2, decodeY( y2 ) );

    window.repaint();
}

/**
 * Draws a rectangle in the graphics window, given its
 * starting and ending point.
 * @param x1 the x coordinate of the starting point
 * @param y1 the y coordinate of the starting point
 * @param x2 the x coordinate of the ending point
 * @param y2 the y coordinate of the ending point
 * @param filled whether to fill the rectangle or not
 */
public static void rectangle(int x1, int y1, int x2, int y2, boolean filled)
{
    final Graphics grfs;
    final int decodedY = decodeY( y2 );

    if (x1 > x2) {
      int t = x1;
      x1 = x2;
      x2 = t;
    }

    if (y1 > y2) {
      int t = y1;
      y1 = y2;
      y2 = t;
    }

    final int rectWidth = Math.abs( x2 - x1 );
    final int rectHeight = Math.abs( y2 - y1 );

    chkOpen();
    grfs = canvas.getGraphics();
    grfs.setColor( fg );

    if ( filled ) {
        grfs.fillRect( x1, decodedY, rectWidth, rectHeight );
    } else {
        grfs.drawRect( x1, decodedY, rectWidth, rectHeight );
    }

	window.repaint();
}

/**
 * Draws a rectangle with rounded edges in the graphics window,
 * given its starting and ending point.
 * @param x1 the x coordinate of the starting point
 * @param y1 the y coordinate of the starting point
 * @param x2 the x coordinate of the ending point
 * @param y2 the y coordinate of the ending point
 * @param arcWidth the width of the arcs for the edges
 * @param arcHeight the height of the arcs for the edges
 */
public static void roundRectangle(int x1, int y1, int x2, int y2,
                int arcWidth, int arcHeight)
{
    final Graphics grfs;

    if (x1 > x2) {
      int t = x1;
      x1 = x2;
      x2 = t;
    }

    if (y1 > y2) {
      int t = y1;
      y1 = y2;
      y2 = t;
    }

    final int rectWidth = Math.abs( x2 - x1 );
    final int rectHeight = Math.abs( y2 - y1 );

    chkOpen();
    grfs = canvas.getGraphics();
    grfs.setColor( fg );
    grfs.drawRoundRect( x2, decodeY( y2 ), rectWidth, rectHeight, arcWidth, arcHeight );
    window.repaint();
}

/**
 * Draws a circle in the graphics window,
 * given its starting and ending point.
 * @param x1 the x coordinate of the starting point
 * @param y1 the y coordinate of the starting point
 * @param x2 the x coordinate of the ending point
 * @param y2 the y coordinate of the ending point
 * @param filled whether to fill the circle or not
 */
public static void circle(int x, int y, int r, boolean filled)
{
    oval( x, y, r, r, filled );
}

/**
 * Draws an oval in the graphics window,
 * given its starting and ending point.
 * @param x1 the x coordinate of the center
 * @param y1 the y coordinate of the center
 * @param dimX the width of the oval
 * @param dimY the height of the oval
 * @param filled whether to fill the oval or not
 */
public static void oval(int x1, int y1, int dimX, int dimY, boolean filled)
{
    final Graphics grfs;
    final int decodedY = decodeY( y1 );

    chkOpen();
    grfs = canvas.getGraphics();
    grfs.setColor( fg );

    if ( filled ) {
        grfs.fillOval( x1 - dimX, decodedY - dimY, dimX * 2, dimY * 2);
    } else {
        grfs.drawOval( x1 - dimX, decodedY - dimY, dimX * 2, dimY * 2);
    }

    window.repaint();
}

/**
 * Draws a point in the graphics window, given its coordinates
 * @param x the x coordinate
 * @param y the y coordinate
 */
public static void point(int x, int y)
{
    circle( x, y, 1, false );
}

/**
 * Draws an arc in the graphics window,
 * given its starting and ending point.
 * The resulting arc begins at startAngle and
 * extends for arcAngle degrees.
 * The left starting point of the arc is the left lower point
 * of the enclosing rectangle.
 * @param x1 the x coordinate of the center of the arc
 * @param y1 the y coordinate of the center of the arc
 * @param arcWidth the width of the arc
 * @param arcHeight the height
 * @param startAngle the angle to start the arc with
 * @param arcAngle the angle of the arc
 */
public static void arc(int x1, int y1,
                       int arcWidth, int arcHeight,
                       int startAngle, int arcAngle)
{
    final Graphics grfs;

    chkOpen();
    grfs = canvas.getGraphics();
    grfs.setColor( fg );
    grfs.drawArc( x1 - arcWidth, decodeY( y1 ) - arcHeight, arcWidth * 2, arcHeight * 2, startAngle, arcAngle );
    window.repaint();
}

/**
 * Draws a polygon, defined by the given coordinates.
 * Note that both vectors must be equal in length.
 * @param xs The x coordinates for all points.
 * @param ys The y coordinates for all points.
 */
public static void polygon(int[] xs, int[] ys)
{
    final Graphics grfs;

    if ( xs.length == ys.length ) {
        chkOpen();

        decodeY( ys );
        grfs = canvas.getGraphics();
        grfs.setColor( fg );
        grfs.drawPolygon( xs, ys, xs.length );
    }

    window.repaint();
}

/**
 * Draws a polyline, defined by the given coordinates.
 * Points are drawn, and then connected.
 * Note that both vectors must be equal in length.
 * @param xs The x coordinates for all points.
 * @param ys The y coordinates for all points.
 */
public static void polyline(int[] xs, int[] ys)
{
    final Graphics grfs;

    if ( xs.length == ys.length ) {
        chkOpen();

        decodeY( ys );
        grfs = canvas.getGraphics();
        grfs.setColor( fg );
        grfs.drawPolyline( xs, ys, xs.length );
    }

    window.repaint();
}

/**
 * Draws a string in the graphic window
 * @param x the x coordinate of the starting point
 * @param y the y coordinate of the starting point
 * @param str the vector of chars containing the string.
 */
public static void print(int x, int y, char[] str)
{
    final Graphics grfs;

    chkOpen();
    grfs = canvas.getGraphics();
    grfs.setColor( fg );
    grfs.drawChars( str, 0, str.length, x, decodeY( y ) );
    window.repaint();
}

public static void setFontStyle(String strFontStyle)
{
	final Graphics grfs;
	String name = "serif";
	int size = 10;
	String fontNames = " mono serif sans-serif ";

	String[] parameters = strFontStyle.split( " " );

	for(int i = 0; i < parameters.length; ++i) {
		String parameter = parameters[ i ].trim();

		parameter = parameter.toLowerCase();
		if ( parameter.endsWith( "px" ) ) {
			try {
				size = Integer.parseInt( parameter.substring( 0, parameter.length() - 2 ) );
			} catch(Exception exc)
			{
				size = 20;
			}
		}
		else
		if ( fontNames.indexOf( parameter ) > -1 ) {
			name = parameter;
		}
	}

    chkOpen();
    grfs = canvas.getGraphics();
    grfs.setFont( new Font( name, Font.PLAIN, size ) );
}

/**
 * Clears the whole graphics window with the white color.
 */
public static void clearScreen()
{
    clearScreen( White );
}

/**
 * Clears the whole graphics window with the given color.
 * @param color the color to fill the graphics window with,
 *        following the list of colors available.
 */
public static void clearScreen(int color)
{
    chkOpen();
    setBgColor( color );
}

/**
 * @return the height of the graphics window.
 */
public static int getHeight()
{
    return height;
}

/**
 * @return the width of the graphics window.
 */
public static int getWidth()
{
    return width;
}

/**
 * Changes the size of the graphics window.
 * The old windows is closed, if existed, and a new one is open.
 * @param w the new width
 * @param h the new height
 */
public static void setSize(int w, int h)
{
	// Adjust
	if ( w < MinWidth ) {
			w = MinWidth;
	}

	if ( h < MinHeight ) {
			h = MinHeight;
	}

	// Set size
	if ( w != width
	  || h != height )
	{
		width = w;
		height = h;
		open();
	} else {
		chkOpen();
	}
}

/**
 * Shows or hides the graphics window.
 * If there is no window, and visible is false, then nothing happens.
 * @param visible sets the visibility of the window
 */
public static void setVisible(boolean visible)
{
    // Open it if doesn't exist and going to be visible
    if ( visible ) {
        chkOpen();
    }

    // If the window exists, apply the change
    if ( window != null ) {
        window.setVisible( visible );
    }

    return;
}

/**
 * Closes the current graphics window.
 */
public static void close()
{
    if ( window != null ) {
        window.setVisible( false );
        canvas = null;
        window = null;
    }

    return;
}

/**
 * Changes the title of the graphics window
 * @param cvTitle the new title
 */
public static void setTitle(char[] cvTitle)
{
    chkOpen();

    title = new String( cvTitle );
    window.setTitle( title );
}

private static void chkOpen()
{
    if ( window == null ) {
        open();
    }

    return;
}

private static void open()
{
    open( width, height );
}

private static void open(int w, int h)
{
    width = w;
    height = h;
    close();

    // Opens a window, ready to draw
    window = new JFrame();
    window.setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );

    // Prepares the canvas to draw through image
    canvas = new BufferedImage( width, height,  BufferedImage.TYPE_INT_RGB);
    JLabel canvasFrame = new JLabel( new ImageIcon( canvas ) );
    canvasFrame.setPreferredSize( new Dimension( width, height ) );
    canvasFrame.setMinimumSize( new Dimension( width, height ) );
    window.getContentPane().add( canvasFrame );

    // Start with colors bg = black/fg = white
	setColors( Black, White );

    // Finish
    window.setTitle( title );
    window.pack();
    window.setVisible( true );
}

private static int width = 320;
private static int height = 240;
private static JFrame window = null;
private static BufferedImage canvas = null;
private static Color fg = Color.white;
private static Color bg = Color.black;
private static String title = "Screen";

} // class gw - jC media library
