
/**
    jC gw media library package
    @name   gw (jC media Library)
    @brief  Common graphic functions
    @date   2010-12-1
    @author jbgarcia@uvigo.es
*/

var gw = ( function() {

	var MinWidth = 100;
	var MinHeight = 100;
	var ScreenDivId = "screen_div";
	var ScreenCanvasId = "screen";

	function removeScreen()
	{
		var screenDiv = document.getElementById( ScreenDivId );

		if ( screenDiv != null ) {
			screenDiv.innerHTML = "";
		} else {
			createScreenFrame();
		}

		return;
	}

	function createScreenFrame()
	{
		var screenDiv = document.getElementById( ScreenDivId );

		if ( screenDiv == null ) {
			screenDiv = document.createElement( "div" );
			screenDiv.setAttribute( "id", ScreenDivId );
			document.body.appendChild( screenDiv );
		}

		return screenDiv;
	}

	function createScreen()
	{
		var screenDiv = document.getElementById( ScreenDivId );
		var scrCanvas = document.getElementById( ScreenCanvasId );

		if ( screenDiv == null ) {
			screenDiv = createScreenFrame();
		}

		if ( scrCanvas != null ) {
			removeScreen();
		}
		
		scrCanvas = document.createElement( "canvas" );
		scrCanvas.setAttribute( "id", ScreenCanvasId );
		scrCanvas.setAttribute( "width", width );
		scrCanvas.setAttribute( "height", height );
		
		screenDiv.appendChild( scrCanvas );
		setColors( Black, White );
		setVisible( true );
		return scrCanvas;
	}

	function chkOpen()
	{
		var scrCanvas = document.getElementById( ScreenCanvasId );
        
        if ( scrCanvas == null ) {
			scrCanvas = createScreen();
        }
        
        return scrCanvas;
	}
	
	function getScreenContext()
	{
		return chkOpen().getContext( "2d" );
	}

	//	Colores:
	var Colors = [
		/* 0 - Black, Negro */ 				"#000000", 
		/* 1 - Blue, Azul */ 				"#0000FF",
		/* 2 - Cyan, Cian */ 				"#00FFFF",
		/* 3 - Gray, Grey, Gris */			"#C0C0C0",
		/* 4 - Green, Verde */				"#00FF00",
		/* 5 - Magenta */					"#FF00FF",
		/* 6 - Orange, Naranja */			"#FF9900",
		/* 7 - Pink, Rosa */				"#FF99FF",
		/* 8 - Red, Rojo */					"#FF0000",
		/* 9 - White, Blanco */				"#FFFFFF",
		/* 10 - Yellow, Amarillo */			"#FFFF00",
		/* 11 - Brown, marrón */ 			"#A52A2A",
		/* 12 - Dark gray, gris oscuro */	"#A0A0A0",
		/* 13 - Light Gray, gris claro */	"#D0D0D0",
	];
	
	var MaxColors = Colors.length;
	
	var Black = 0;
	var Blue = 1;
	var Cyan = 2;
	var Gray = 3;
	var Green = 4;
	var Magenta = 5;
	var Orange = 6;
	var Pink = 7;
	var Red = 8;
	var White = 9;
	var Yellow = 10;
	var Brown = 11;
	var DarkGray = 12;
	var LightGray = 13;
	
	/**
	 * iJava sets the beginning of the coordinates at the lower left corner.
	 * This means that the Y (vertical) coordinate must be decoded accordingly.
	 * @param y The Y coordinate in the iJava graphics screen.
	 * @return The decodified Y coordinate.
	 */
	function decodeY(y)
	{
		var toret;
		
		if ( y instanceof Array ) {
			toret = y.slice( 0 );
			for(var i = 0; i < toret.length; ++i) {
				toret[ i ] = Math.abs( getHeight() - y[ i ] - 1 );
			}
		} else {			
			toret = Math.abs( getHeight() - y - 1 );
		}
		
		return toret;
	}
	
	/**
	 * Converts a color from the above list, to a
	 * Color object from the Java's AWT system.
	 * @param c the color, as an int from black (0) to yellow(10)
	 * @param r the code color for red component
	 * @param g the code color for green component
	 * @param b the code color for blue component
	 * @return the corresponding color, as a Color object.
	 */
	function decodeColor(c)
	{
		var toret = Colors[ Black ];
		
		if ( arguments.length == 1 ) {
			if ( c === null
			  || c === undefined )
			{
				c = 0;
			}
			else
			if ( c < 0
			  || c >= MaxColors )
			{
				c = 0;
			}

			toret = Colors[ Math.round( c ) ];
		}
		else
		if ( arguments.length == 3 ) {
			toret = "#\"";

			for(var i = 0; i < arguments.length; ++i) {
				if ( arguments[ i ] === null
				  || arguments[ i ] === undefined )
				{
					arguments[ i ] = 0;
				} else {
					arguments[ i ] = Math.round( arguments[ i ] );
				}
			}

			toret += ( "0" + arguments[ 0 ].toString( 16 ) ).substr( -2 );
			toret += ( "0" + arguments[ 1 ].toString( 16 ) ).substr( -2 );
			toret += ( "0" + arguments[ 2 ].toString( 16 ) ).substr( -2 );
			toret += '"';
		}
		
		return toret;
	}
	
	/**
	 * Changes the color used for the foreground
	 * @param color as an int, indexing the list of colors
	 * @param r the red component, as int
	 * @param g the green component, as int
	 * @param b the blue component, as int
	 */
	function setFgColor(color)
	{
		chkOpen();
		
		fg = Colors[ Black ];
		
		if ( arguments.length == 1 ) {
			fg = decodeColor( color );
		}
		else
		if ( arguments.length == 3 ) {
			fg = decodeColor( arguments[ 0 ], arguments[ 1 ], arguments[ 2 ] );
		}

		return;
	}
	
	/**
	 * Changes the colors used for foreground and background.
	 * @param fg the new color for the foreground, as int
	 * @param bg the new color for the background, as int
	 */
	function setColors(f, b)
	{
		setBgColor( b );
		setFgColor( f );
	}
	
	/**
	 * Changes the color used for the background
	 * @parameter color as an int, indexing the list of colors
	 */
	function setBgColor(bgc)
	{
		var ctx = getScreenContext();

		// Set color
		bg = decodeColor( bgc );

		// Clear
		ctx.beginPath();
		ctx.rect( 0, 0, getWidth(), getHeight() );
		ctx.fillStyle = bg;
		ctx.strokeStyle = bg;
		ctx.fill();
		ctx.stroke();
	}
	
	/**
	 * Draws a line in the graphics window, given its
	 * starting and ending point.
	 * @param x1 the x coordinate of the starting point
	 * @param y1 the y coordinate of the starting point
	 * @param x2 the x coordinate of the ending point
	 * @param y2 the y coordinate of the ending point
	 */
	function line(x1, y1, x2, y2)
	{
		var ctx = getScreenContext();

		ctx.beginPath();
		ctx.moveTo( x1, decodeY( y1 ) );
		ctx.lineTo( x2, decodeY( y2 ) );
		ctx.strokeStyle = fg;
		ctx.stroke();
	}
	
	/**
	 * Draws a rectangle in the graphics window, given its
	 * starting and ending point.
	 * @param x1 the x coordinate of the starting point
	 * @param y1 the y coordinate of the starting point
	 * @param x2 the x coordinate of the ending point
	 * @param y2 the y coordinate of the ending point
	 * @param filled whether to fill the rectangle or not
	 */
	function rectangle(x1, y1, x2, y2, filled)
	{
		var ctx = getScreenContext();
		var decodedY = decodeY( y2 );
		var rectWidth = Math.abs( x2 - x1 );
		var rectHeight = Math.abs( y2 - y1 );

		// Invert, if needed.
		if ( x1 > x2 ) {
		  var t = x1;
		  x1 = x2;
		  x2 = t;
		}

		if ( y1 > y2 ) {
		  var t = y1;
		  y1 = y2;
		  y2 = t;
		}

		// Draw
		ctx.beginPath();
		ctx.rect( x1, decodedY, rectWidth, rectHeight );
		if ( filled ) {
			ctx.fillStyle = fg;
			ctx.fill();
		}

		// Show
		ctx.strokeStyle = fg;
		ctx.stroke();
	}
	
	/**
	 * Draws a rectangle in the graphics window, given its
	 * starting and ending point.
	 * @param x1 the x coordinate of the starting point
	 * @param y1 the y coordinate of the starting point
	 * @param x2 the x coordinate of the ending point
	 * @param y2 the y coordinate of the ending point
	 * @param arcWidth width of the arc for the angles
	 * @param arcHeight height of the arc for the angles
	 */
	function roundRectangle(x1, y1, x2, y2, arcWidth, arcHeight)
	{
		var ctx = getScreenContext();
		var radius = arcWidth / 2;
		var width = Math.abs( x2 - x1 );
		var height = Math.abs( y2 - y1 );

		y1 = decodeY( y1 );
		y2 = decodeY( y2 );
		
		// Invert, if needed.
		if ( x1 > x2 ) {
		  var t = x1;
		  x1 = x2;
		  x2 = t;
		}

		if ( y1 > y2 ) {
		  var t = y1;
		  y1 = y2;
		  y2 = t;
		}

		// Draw
		ctx.beginPath();
		
		ctx.moveTo( x1 + radius, y1 );
		ctx.lineTo( x1 + width - radius, y1 );
		ctx.quadraticCurveTo( x1 + width, y1, x1 + width, y1 + radius );
		ctx.lineTo( x1 + width, y1 + height - radius );
		ctx.quadraticCurveTo( x1 + width, y1 + height, x1 + width - radius, y1 + height );
		ctx.lineTo( x1 + radius, y1 + height );
		ctx.quadraticCurveTo( x1, y1 + height, x1, y1 + height - radius );
		ctx.lineTo( x1, y1 + radius );
		ctx.quadraticCurveTo( x1, y1, x1 + radius, y1 );
		ctx.closePath();

		// Show
		ctx.strokeStyle = fg;
		ctx.stroke();
	}

	/**
	 * Draws a circle in the graphics window,
	 * given its starting and ending point.
	 * @param x1 the x coordinate of the starting point
	 * @param y1 the y coordinate of the starting point
	 * @param x2 the x coordinate of the ending point
	 * @param y2 the y coordinate of the ending point
	 * @param filled whether to fill the circle or not
	 */
	function circle(x, y, r, filled)
	{
		oval( x, y, r, r, filled );
	}

	function oval(x1, y1, dimX, dimY, filled)
	{
		var ctx = getScreenContext();
		var y1 = decodeY( y1 );

		// Draw ellipse
		ctx.save();
		ctx.scale( 1, dimY / dimX );
		ctx.beginPath();
		ctx.arc( x1, y1, dimX, 0, Math.PI * 2, false );

		if ( filled ) {
			ctx.fillStyle = fg;
			ctx.fill();
		}

		// Show
		ctx.closePath();
		ctx.strokeStyle = fg;
		ctx.stroke();
		ctx.restore();
	}

	/**
	 * Draws a point in the graphics window, given its coordinates
	 * @param x the x coordinate
	 * @param y the y coordinate
	 */
	function point(x, y)
	{
		circle( x, y, 1, false );
	}

	/**
	 * Draws an arc in the graphics window,
	 * given its starting and ending point.
	 * The resulting arc begins at startAngle and
	 * extends for arcAngle degrees.
	 * The left starting point of the arc is the left lower point
	 * of the enclosing rectangle.
	 * @param x1 the x coordinate of the center of the arc
	 * @param y1 the y coordinate of the center of the arc
	 * @param arcWidth the width of the arc
	 * @param arcHeight the height
	 * @param startAngle the angle to start the arc with
	 * @param arcAngle the angle of the arc
	 */
	function arc(x1, y1, arcWidth, arcHeight, startAngle, arcAngle)
	{
		var ctx = getScreenContext();

		// Prepare		
		y1 = decodeY( y1 );
		startAngle = startAngle * ( Math.PI / 180 );		// Radians
		arcAngle = arcAngle * ( Math.PI / 180 );			// Radians

		// Draw ellipse
		ctx.save();
		ctx.scale( 1, arcHeight / arcWidth );
		ctx.beginPath();
		ctx.arc( x1, y1, arcWidth / 2, startAngle, startAngle + arcAngle, false );

		// Show
		ctx.strokeStyle = fg;
		ctx.stroke();
		ctx.restore();
	}

	/**
	 * Draws a polygon, defined by the given coordinates.
	 * Note that both vectors must be equal in length.
	 * @param xs The x coordinates for all points.
	 * @param ys The y coordinates for all points.
	 */
	function polygon(xs, ys)
	{
		var ctx = getScreenContext();
		var decodedYS = decodeY( ys );

		if ( decodedYS.length > 0 ) {
			// Draw polygon
			ctx.beginPath();
			ctx.moveTo( xs[ 0 ], decodedYS[ 0 ] );
			for(var i = 1; i < decodedYS.length; ++i) {
				ctx.lineTo( xs[ i ], decodedYS[ i ] );
			}
			

			// Show
			ctx.closePath();
			ctx.strokeStyle = fg;
			ctx.stroke();
		}

		return;
	}

	/**
	 * Draws a polygon, defined by the given coordinates.
	 * Note that both vectors must be equal in length.
	 * @param xs The x coordinates for all points.
	 * @param ys The y coordinates for all points.
	 */
	function polyline(xs, ys)
	{
		var ctx = getScreenContext();
		var decodedYS = decodeY( ys );

		if ( decodedYS.length > 0 ) {
			// Draw polyline
			ctx.beginPath();
			ctx.moveTo( xs[ 0 ], decodedYS[ 0 ] );
			for(var i = 1; i < decodedYS.length; ++i) {
				ctx.lineTo( xs[ i ], decodedYS[ i ] );
			}
			

			// Show
			ctx.strokeStyle = fg;
			ctx.stroke();
		}

		return;
	}

	/**
	 * Changes the font to use for printing
	 * @param strFontStyle A string in the form "<size>px <font>"
	 * 					   as in "12px mono"
	 */
	function setFontStyle(strFontStyle)
	{
		getScreenContext().font = strFontStyle;
	}

	/**
	 * Draws a string in the graphic window
	 * @param x the x coordinate of the starting point
	 * @param y the y coordinate of the starting point
	 * @param str the vector of chars containing the string.
	 */
	function print(x, y, str)
	{
		var ctx = getScreenContext()

		ctx.strokeStyle = fg;
		ctx.fillStyle = fg;
		ctx.fillText( str, x, decodeY( y ) );
	}

	/**
	 * Clears the whole graphics window with the white color.
	 */
	function clearScreen(color)
	{
		if ( arguments.length < 1 ) {
			color = White;
		}
		
		setBgColor( color );
	}

	/**
	 * Changes the size of the graphics window.
	 * The old window is closed, if existed, and a new one is open.
	 * @param w the new width
	 * @param h the new height
	 */
	function setSize(w, h)
	{
		// Adjust
		if ( w < MinWidth ) {
				w = MinWidth;
		}

		if ( h < MinHeight ) {
				h = MinHeight;
		}

		// Set size
		if ( w != width
		  || h != height )
		{
			width = w;
			height = h;
			createScreen();
		} else {
			chkOpen();
		}

		return;
	}

	/**
	 * Shows or hides the graphics window.
	 * If there is no window, and visible is false, then nothing happens.
	 * @param visible sets the visibility of the window
	 */
	function setVisible(visible)
	{
		// Open it if doesn't exist and going to be visible
		if ( visible ) {
			chkOpen();
		}

		// If the canvas exists, apply the change
		var scrCanvas = document.getElementById( ScreenCanvasId );
		if ( scrCanvas != null ) {
			scrCanvas.style.display = visible ? "block" : "none";
		}

		return;
	}

	/** Closes the canvas */
	function close()
	{
		chkOpen();
		removeScreen();
	}

	/**
	 * Changes the title of the graphics window
	 * @param title the new title
	 */
	function setTitle(title)
	{
		document.title = title;
	}
	
	function getHeight()
	{
		return height;
	}
	
	function getWidth()
	{
		return width;
	}
	
	var fg = Colors[ Black ];
	var bg = Colors[ White ];
	
	var width = 320;
	var height = 240;

	return {
		"MinWidth": MinWidth,
		"MinHeight": MinHeight,
		"ScreenDivId": ScreenDivId,
		"ScreenCanvasId": ScreenDivId,
		"Colors": Colors,
		"MaxColors": MaxColors,
		"Black": Black,
		"Negro": Black,
		"Blue": Blue,
		"Azul": Blue,
		"Cyan": Cyan,
		"Cian": Cyan,
		"Gray": Gray,
		"Grey": Gray,
		"Gris": Gray,
		"Green": Green,
		"Verde": Green,
		"Magenta": Magenta,
		"Orange": Orange,
		"Naranja": Orange,
		"Pink": Pink,
		"Rosa": Pink,
		"Red": Red,
		"Rojo": Red,
		"White": White,
		"Blanco": White,
		"Yellow": Yellow,
		"Amarillo": Yellow,
		"Brown": Brown,
		"Marron": Brown,
		"DarkGray": DarkGray,
		"DarkGrey": DarkGray,
		"GrisOscuro": DarkGray,
		"LightGray": LightGray,
		"LightGrey": LightGray,
		"GrisClaro": LightGray,
		"decodeY": decodeY,
		"decodeColor": decodeColor,
		"setFgColor": setFgColor,
		"setColor": setFgColor,
		"setColors": setColors,
		"setBgColor": setBgColor,
		"line": line,
		"getHeight": getHeight,
		"getWidth": getWidth,
		"rectangle": rectangle,
		"roundRectangle": roundRectangle,
		"circle": circle,
		"oval": oval,
		"point": point,
		"arc": arc,
		"polygon": polygon,
		"polyline": polyline,
		"print": print,
		"setFontStyle": setFontStyle,
		"clearScreen": clearScreen,
		"setSize": setSize,
		"setVisible": setVisible,
		"close": close,
		"setTitle": setTitle,
	};
})();
