/**
    @name   Char management (jC Standard Library)
    @brief  Common operations over characters
    @date   2010-12-1
    @author jbgarcia@uvigo.es
*/

public class charType {

/// Characters considered as punctuation marks
public static final char[] PunctuationChars = new char[]{
    '.', ',', ';', ':', '-', '(', ')'
};
public static final char[] BlankChars = new char[] { ' ', '\t' };

/**
	@brief Decides whether the char is letter
	@param ch The char to check
	@return true if it is letter; false if not

*/

public static boolean isLetter(char ch)
{
	return Character.isLetter( ch );
}

/**
	@brief Decides whether the char is digit or letter
	@param ch The char to check
	@return true if it is letter; false if not

*/

public static boolean isDigit(char ch)
{
	return Character.isDigit( ch );
}

/**
	@brief Decides whether the char is digit or letter
	@param ch The char to check
	@return true if it is letter or digit; false otherwise

*/

public static boolean isAlphaNumeric(char ch)
{
	return ( isDigit( ch ) || isLetter( ch ) );
}

/**
	@brief Decides whether the char is a control character
	@param ch The char to check
	@return true if it is a basic letter or digit; false otherwise

*/

public static boolean isControl(char ch)
{
	return Character.isISOControl( ch );
}

/**
	@brief Decides whether the char is lower case
	@param ch The char to check
	@return true if it is lowercase; false if not
*/

public static boolean isLowerCase(char ch)
{
	return Character.isLowerCase( ch );
}

/**
	@brief Decides whether the char is upper case
	@param ch The char to check
	@return true if it is lowercase; false if not
*/

public static boolean isUpperCase(char ch)
{
	return Character.isUpperCase( ch );
}

/**
	@brief Checks if the character is a delimiter
	@param ch The character to check
	@return true if ch is tab (various forms) or space, false otherwise.
*/
public static boolean isSpace(char ch)
{
	return ( new String( BlankChars ).indexOf( ch ) >= 0 );
}

/**
	@brief Checks if the character is punctuation mark
	@param ch The character to check
	@see PunctuationChars
	@return true if ch is punctuation, false otherwise.
*/
public static boolean isPunctuation(char ch)
{

	return ( new String( PunctuationChars ).indexOf( ch ) >= 0 );
}

/**
	@brief Changes the char to lower case
	@param ch The char to change
	@return The converted char
*/

public static char toLowerCase(char ch)
{
	return Character.toLowerCase( ch );
}

/**
	@brief Changes the char to upper case
	@param ch The char to change
	@return The converted char
*/

public static char toUpperCase(char ch)
{
	return Character.toUpperCase( ch );
}

} // class charType jC std lib
