
import javax.swing.*;
import java.lang.reflect.Array;


/**
    jC io standard library package
    @name   Console Input/Output (jC Standard Library)
    @brief  Common read/write operations on console
    @date   2010-12-1
    @author jbgarcia@uvigo.es
*/

public class io {

/**
	@brief Prints anything on console
	@param s Any variable of any type
	@hide
*/

public static void write(Object s)
{
	if ( s.getClass().isArray()
	  && Array.get( s, 0 ) instanceof Character )
	{
		System.out.print( new String( (char[]) s ) );
	} else {
		System.out.print( s.toString() );
	}
}

/**
	@brief Prints anything on console, and changes line
	@param s Any variable of any type
	@hide
*/

public static void writeln(Object s)
{
    write( s );
	System.out.println();
}

/**
	@brief Changes to next line on console
*/

public static void writeln()
{
	System.out.println();
}

public static void print(Object s)
{
	write( s );
}

/**
	@brief Prints anything on console, and changes line
	@param s Any variable of any type
	@hide
*/

public static void println(Object s)
{
	writeln( s );
}

/**
	@brief Changes to next line on console
*/

public static void println()
{
	System.out.println();
}

/**
	Reads a string from console
	@return A vector-formed string
*/
public static char[] readln()
{
	return readln( "Input data: ".toCharArray() );
}

/**
	Reads a string from console
	@return A vector-formed string
*/
public static char[] readString()
{
	return readln( "Input data: ".toCharArray() );
}

/**
	Reads a string from console
	@param msg A message to show just before input happens
	@return A vector-formed string
*/
public static char[] readln(char[] msg)
{
	write( msg );

	String line = (String) JOptionPane.showInputDialog(
            null,
		    new String( msg ),
			"Input data",
			JOptionPane.INFORMATION_MESSAGE,
		    null,
			null,
            ""
	);

	if ( line == null ) {
		line = "";
	}

	writeln( line );
	return line.toCharArray();
}

/**
	Reads an integer from console
	@return An integer
*/
public static int readInteger()
{
	int toret = 0;
	char[] msg = "Input an integer value: ".toCharArray();
	String line;
	boolean conversionError;
	
	do {
		conversionError = false;
		line = new String( readln( msg ) );

		if ( line == null ) {
				line = "";
		}

		try {
				toret = Integer.parseInt( line );
		} catch(Exception exc)
		{
				conversionError = true;
		}
	} while( conversionError );
	
	return toret;
}

/**
	Reads a double from console
	@return A double
*/
public static double readDouble()
{
	double toret = 0;
	char[] msg = "Input a double value: ".toCharArray();
	String line;
	boolean conversionError;
	
	do {
		conversionError = false;
		line = new String( readln( msg ) );

		if ( line == null ) {
				line = "";
		}

		try {
				toret = Double.parseDouble( line );
		} catch(Exception exc)
		{
				conversionError = true;
		}
	} while( conversionError );
        
	return toret;
}

/**
	Reads a char from console
	@return A char
*/
public static char readChar()
{
	char[] msg = "Input a char value: ".toCharArray();
	String line;
	boolean conversionError;
	
	do {
			conversionError = false;
			line = new String( readln( msg ) );

			if ( line == null ) {
					line = "";
			}
	
			if ( line.length() != 1 ) {
					conversionError = true;
			}                
	} while( conversionError );
        
	return line.charAt( 0 );
}

} // class io - jC standard library
