/**
    jC io standard library package
    @name   Console Input/Output (jC Standard Library)
    @brief  Common read/write operations on console
    @date   2013-7-15
    @author jbgarcia@uvigo.es
*/

var io = ( function() {
	
    var OutputDivId = "output";
    
    function prepareOutput()
    {
        var outputDiv = document.getElementById( OutputDivId );
        
        if ( outputDiv == null ) {
            outputDiv = document.createElement( "div" );
            outputDiv.setAttribute( "id", OutputDivId );
            document.body.appendChild( outputDiv );
        }
        
        return outputDiv;
    }
	
    /**
        @brief Prints anything on console
        @param x Any variable of any type
    */
    function write(x)
    {
        var outputDiv = prepareOutput();
        var lastChild = outputDiv.lastChild;
        
	if ( Object.prototype.toString.call( x ) === '[object Array]'
          && typeof( x[ 0 ] ) === 'string' )
	{
		x = x.join( "" );
	}

        if ( lastChild == null ) {
            outputDiv.appendChild( document.createTextNode( x ) );
        } else {
            lastChild.appendData( x );
        }
        
        return;
    }
    
    /**
        @brief Prints anything on console, and changes line
        @param x Any variable of any type
    */
    function writeln(x)
    {
        if ( arguments.length > 0 ) {
            write( x );
        }
        
        var outputDiv = prepareOutput();
        outputDiv.appendChild( document.createElement( "br" ) );
        outputDiv.appendChild( document.createTextNode( "" ) );
        return;
    }
    
    /**
		Reads a string from console
		@param msg A message to show just before input happens
		@return A vector-formed string
	*/
    function readln(msg)
    {
		var txt = "Input data: ";
		
		if ( arguments.length > 0 ) {
			txt = msg;
		}
		
		write( txt );
		
		var toret = window.prompt( txt, "" );
		
		if ( toret == null ) {
			toret = "";
		}
		
		writeln( toret );
		return toret;
	}
	
	/**
		Reads an integer from console
		@return An integer
	*/
	function readInteger()
	{
		var toret = 0;
		var msg = "Input an integer number: ";
		
		do {
			toret = parseInt( readln( msg ) );
		} while( isNaN( toret ) );
		
		return toret;
	}
	
	/**
		Reads a double from console
		@return A double
	*/
	function readDouble()
	{
		var toret = 0;
		var msg = "Input a real number: ";
		
		do {
			toret = parseFloat( readln( msg ) );
		} while( isNaN( toret ) );
		
		return toret;
	}
	
	/**
		Reads a char from console
		@return A char
	*/
	function readChar()
	{
		var toret = "";
		var msg = "Input a character: ";
		
		do {
			toret = readln( msg );
		} while( toret.length != 1 );
		
		return toret.substr( 0, 1 );
	}
	
	return {
		"OutputDivId": OutputDivId,
        "write": write,
        "writeln": writeln,        
		"print": write,
        "println": writeln,
        "readln": readln,
        "readString": readln,
        "readInteger": readInteger,
        "readDouble": readDouble,
        "readChar": readChar,
	};
})();
