
/**
    @name   Math (jC Standard Library)
    @brief  Common mathematical functions
    @date   2010-12-1
    @author jbgarcia@uvigo.es
*/

var math = ( function() {
    
    // For the number generator
    var seed = null;
    
	/// The PI constant
	var PI = Math.PI;

	/// The e constant
	var E = Math.E;

	/**
		@brief Absolute value function
		@param x the value to apply this function
		@return the result of this function applied to x
	*/
	function abs(x)
	{
		return Math.abs( x );
	}
	
	/**
		@brief Cosene function
		@param x the value to apply this function
		@return the result of this function applied to x
	*/
	function cos(x)
	{
		return Math.cos( x );
	}
	
	/**
		@brief Sine function
		@param x the value to apply this function
		@return the result of this function applied to x
	*/
	function sin(x)
	{
		return Math.sin( x );
	}
	
	/**
		@brief Arc cosene function
		@param x the value to apply this function
		@return the result of this function applied to x
	*/

	function acos(x)
	{
		return Math.acos( x );
	}

	/**
		@brief Arc sine function
		@param x the value to apply this function
		@return the result of this function applied to x
	*/
	function asin(x)
	{
		return Math.asin( x );
	}

	/**
		@brief Tangential function
		@param x the value to apply this function
		@return the result of this function applied to x
	*/
	function tan(x)
	{
		return Math.tan( x );
	}

	/**
		@brief Arc tangential function
		@param x the value to apply this function
		@return the result of this function applied to x
	*/
	function atan(x)
	{
		return Math.atan( x );
	}	
	
	/**
		@brief Euler's number e raised to the power of x
		@param x the value to apply this function
		@return the result of this function applied to x
	*/
	function exp(x)
	{
		return Math.exp( x );
	}

	/**
		@brief Returns the natural logarithm (base e) of x
		@param x the value to apply this function
		@return the result of this function applied to x
	*/
	function log(x)
	{
		return Math.log( x );
	}

	/**
		@brief Returns the base 10 logarithm of x
		@param x the value to apply this function
		@return the result of this function applied to x
	*/
	function log10(x)
	{
		return ( Math.log( x ) / Math.LN10 );
	}
	
	/**
		@brief Returns the integer number that is smaller than x
		@param x the value to apply this function
		@return the result of this function applied to x (as double)
	*/
	function floor(x)
	{
		return Math.floor( x );
	}

	/**
		@brief Returns the integer number that is bigger than x
		@param x the value to apply this function
		@return the result of this function applied to x (as double)
	*/
	function ceil(x)
	{
		return Math.ceil( x );
	}

	/**
		@brief Returns the a number x to the y-th power
		@param x the value to apply this function
		@param y the exponent
		@return the result of x^y
	*/
	function pow(x, y)
	{
		return Math.pow( x, y );
	}

	/**
		@brief Returns the square root
		@param x the value to apply this function
		@return the result of square root of x
	*/
	function sqrt(x)
	{
		return Math.sqrt( x );
	}

	/**
		@brief Returns the closest integer value
		@param x the value to apply this function
		@return the result as int
	*/
	function round(x)
	{
		return Math.round( x );
	}

	/**
		@brief Returns a number in the range [0, 1)
		@return a random number
	*/
	function random()
	{
		if ( seed === null ) {
			seed = new Date().getTime() % 32749;
		}
		
        seed = ( ( seed * 32719 ) + 3 ) % 32749;
        return ( ( seed % 10000 ) + 1 ) / 10000;
	}
	
	/**
		 @brief Sets the seed of the random generator
		 @param seed The new seed.
	*/
	function srand(x)
	{
		seed = x;
	}
	
	/**
		@brief Returns a conversion of x to degrees
		@param x the value to apply this function
		@return the result in degrees
	*/
	function toDegrees(x)
	{
		return ( x * 180 ) / PI;
	}

	/**
		@brief Returns the conversion of x to radians
		@param x the value to apply this function
		@return the result in degrees
	*/
	function toRadians(x)
	{
		return ( x * PI ) / 180;
	}

	
	return {
		"PI": PI,
		"E": E,
		"abs": abs,
		"cos": cos,
		"sin": sin,
		"acos": acos,
		"asin": asin,
		"tan": tan,
		"atan": atan,
		"exp": exp,
		"log": log,
		"log10": log10,
		"floor": floor,
		"ceil": ceil,
		"round": round,
		"pow": pow,
		"sqrt": sqrt,
		"random": random,
		"rand": random,
		"srand": srand,
		"toDegrees": toDegrees,
		"toRadians": toRadians,
	};
})();
