/**
    jC string standard library package
    @name   String (jC Standard Library)
    @brief  Common string operations
    @date   2010-12-1
    @author jbgarcia@uvigo.es
*/

public class string {

// ------------------------------------------------------------ Basic functions

/**
	@brief Copies an existing string to a new string
	@param s An existing string
	@return A new string as a vector of chars
*/

public static char[] strCopy(char[] s)
{
	return new String( s ).toCharArray();
}

/**
	@brief Creates a new string, concatening two existing strings
	@param s1 An existing string
	@param s2 An existing string
	@return A new string as a concatenation of s1 and s2
*/
public static char[] strConcat(char[] s1, char[] s2)
{
	String toret = new String( s1 );

	toret += new String( s2 );

	return toret.toCharArray();
}

/**
	@brief Compares a string to another one (lexicographically)
	@param s1 An existing string
	@param s2 An existing string
	@return negative value if s1 < s2, 0 if s1 == s2, positive if s1 > s2
*/

public static int strCompare(char[] s1, char[] s2)
{
	String str1 = new String( s1 );
	String str2 = new String( s2 );

	return ( str1.compareTo( str2 ) );
}

/**
	@brief Determines whether a given string is equal to another one.
	@param s1 An existing string
	@param s2 An existing string
	@return true if equals, false otherwise
*/

public static boolean strEq(char[] s1, char[] s2)
{
	return ( strCompare( s1, s2 ) == 0 );
}

/**
	Returns the position of ch in s
	@param s The string to look in
	@param ch The character to look for
	@return -1 if not found; the position otherwise
*/
public static int strIndexOf(char[] s, char ch)
{
	return ( new String( s ).indexOf( ch ) );
}

/**
	Returns the last position of ch in s
	@param s The string to look in
	@param ch The character to look for
	@return -1 if not found; the position otherwise
*/

public static int strLastIndexOf(char[] s, char ch)
{
	return ( new String( s ).lastIndexOf( ch ) );
}

// ------------------------------------------------------- Conversion functions

/**
	@brief Converts a string to an int number
	@param s The string to convert
	@return The number contained; 0 if error.
*/

public static int strToInt(char[] s)
{
	int toret = 0;

	try {
		toret = Integer.parseInt( new String( s ).trim() );
	}
	catch(Exception e) {
	}

	return toret;
}

/**
	@brief Converts a string to a double number
	@param s The string to convert
	@return The number contained; 0 if error.
*/

public static double strToDbl(char[] s)
{
	double toret = 0;

	try {
		toret = Double.parseDouble( new String( s ).trim() );
	}
	catch(Exception e) {
	}

	return toret;
}

/**
	@brief Converts an int number to string
	@param x The number to convert
	@return The number s a string
*/

public static char[] intToStr(int x)
{
	String toret = "";

	try {
		toret = Integer.toString( x );
	}
	catch(Exception e) {
	}

	return toret.toCharArray();
}

/**
	@brief Converts an int number to string
	@param x The number to convert
	@return The number s a string
*/

public static char[] dblToStr(double x)
{
	String toret = "";

	try {
		toret = Double.toString( x );
	}
	catch(Exception e) {
	}

	return toret.toCharArray();
}

/**
	@brief Converts a char to string
	@param ch The char to convert
	@return The char ch as a string
*/

public static char[] charToStr(char ch)
{
	char[] toret = new char[1];

	toret[ 0 ] = ch;

	return toret;
}

/**
	@brief Converts a str to lower case
	@param str The string to convert
	@return The string in lower case
*/

public static char[] strToLowerCase(char[] str)
{
	for(int i = 0; i < str.length; ++i) {
		str[ i ] = Character.toLowerCase( str[ i ] );
	}

	return str;
}

/**
	@brief Converts a str to upper case
	@param str The string to convert
	@return The string in upper case
*/

public static char[] strToUpperCase(char[] str)
{
	for(int i = 0; i < str.length; ++i) {
		str[ i ] = Character.toUpperCase( str[ i ] );
	}

	return str;
}

} // string class

