
/**
    jC string standard library package
    @name   String (jC Standard Library)
    @brief  Common string operations
    @date   2010-12-1
    @author jbgarcia@uvigo.es
*/

var string = ( function() {

	/**
		@brief Copies an existing string to a new string
		@param s An existing string
		@return A new string as a vector of chars
	*/
	function strCopy(s)
	{
		return new String( s );
	}
	
	/**
		@brief Creates a new string, concatening two existing strings
		@param s1 An existing string
		@param s2 An existing string
		@return A new string as a concatenation of s1 and s2
	*/
	function strConcat(s1, s2)
	{
		var toret = new String( s1 );

		toret += new String( s2 );

		return toret;
	}
	
	/**
		@brief Compares a string to another one (lexicographically)
		@param s1 An existing string
		@param s2 An existing string
		@return negative value if s1 < s2, 0 if s1 == s2, positive if s1 > s2
	*/
	function strCompare(s1, s2)
	{
		var str1 = new String( s1 );
		var str2 = new String( s2 );
		var toret = 0;
		
		if ( str1 > str2 ) {
			toret = 1;
		}
		else
		if ( str1 < str2 ) {
			toret = -1;
		}

		return toret;
	}
	
	/**
		@brief Determines whether a given string is equal to another one.
		@param s1 An existing string
		@param s2 An existing string
		@return true if equals, false otherwise
	*/
	function strEq(s1, s2)
	{
		return ( strCompare( s1, s2 ) == 0 );
	}
	
	/**
		Returns the position of ch in s
		@param s The string to look in
		@param ch The character to look for
		@return -1 if not found; the position otherwise
	*/
	function strIndexOf(s, ch)
	{
		return ( new String( s ).indexOf( ch ) );
	}
	
	/**
		Returns the last position of ch in s
		@param s The string to look in
		@param ch The character to look for
		@return -1 if not found; the position otherwise
	*/
	function strLastIndexOf(s, ch)
	{
		return ( new String( s ).lastIndexOf( ch ) );
	}
	
	/**
		@brief Converts a string to an int number
		@param s The string to convert
		@return The number contained; 0 if error.
	*/
	function strToInt(s)
	{
		return parseInt( new String( s ) );
	}
	
	/**
		@brief Converts a string to a double number
		@param s The string to convert
		@return The number contained; 0 if error.
	*/
	function strToDbl(s)
	{
		return parseFloat( new String( s ) );
	}
	
	/**
		@brief Converts an int number to string
		@param x The number to convert
		@return The number s a string
	*/
	function intToStr(x)
	{
		return new String( x );
	}
	
	/**
		@brief Converts an int number to string
		@param x The number to convert
		@return The number s a string
	*/
	function dblToStr(x)
	{
		return new String( x );
	}
	
	/**
		@brief Converts a char to string
		@param ch The char to convert
		@return The char ch as a string
	*/
	function charToStr(x)
	{
		return new String( x );
	}
	
	/**
		@brief Converts a str to lower case
		@param str The string to convert
		@return The string in lower case
	*/
	function strToLowerCase(str)
	{
		return str.toLowerCase();
	}

	/**
		@brief Converts a str to upper case
		@param str The string to convert
		@return The string in upper case
	*/
	function strToUpperCase(str)
	{
		return str.toUpperCase();
	}
	
	return {
		"strCopy": strCopy,
		"strConcat": strConcat,
		"strCompare": strCompare,
		"strEq": strEq,
		"strIndexOf": strIndexOf,
		"strLastIndexOf": strLastIndexOf,
		"strToInt": strToInt,
		"strToDbl": strToDbl,
		"intToStr": intToStr,
		"dblToStr": dblToStr,
		"charToStr": dblToStr,
		"strToLowerCase": strToLowerCase,
		"strToUpperCase": strToUpperCase,
		"charToStr": dblToStr,
	};
})();
