/**
    jC util standard library package
    @name   Util (jC Standard Library)
    @brief  Common convenience functions
    @date   2010-12-1
    @author jbgarcia@uvigo.es
*/

import java.lang.Exception;
import java.util.Calendar;

public class util {

/// Program termination code when an error happened
public static final int ExitFailure = 1;

/// Program termination code for a successful program
public static final int ExitSuccess = 0;

/// Number of seconds in a day 24 * 60 * 60
public static final int SecondsInDay =  86400;

/// Number of minutes in a day
public static final int MinutesInDay = 1440;

/// The number of months in a year
public static final int MonthsInYear = 12;

/// @hide
public static int size(Object[] vector)
{
	return vector.length;
}

/**
	@brief returns the size of a vector
	@param vector The vector
	@return the size as an int
*/

public static int size(char[] vector)
{
	return vector.length;
}

/**
	@brief returns the size of a vector
	@param vector The vector
	@return the size as an int
*/

public static int size(int[] vector)
{
	return vector.length;
}

/**
	@brief returns the size of a vector
	@param vector The vector
	@return the size as an int
*/

public static int size(double[] vector)
{
	return vector.length;
}

/**
	@brief returns whether the size of a vector is zero
	@param vector The vector
	@return true if size equals 0, false otherwise
	@see size
*/

public static boolean isEmpty(Object[] vector)
{
	return size( vector ) == 0;
}

/**
	@brief Assert a condition, stop the program if not true. Exits with error code -1
	@param expr The condition to check
	@param msg The message to show
	@return If expr is false, it never returns
*/

public static void verify(boolean expr, char[] msg)
{
	if ( !expr ) {
		System.err.println( new String( msg ) );
	}

	return;
}

/**
	@brief Returns the current date in the system
	@return Date as a vector of three ints: 0=year,1=month,2=day
*/

public static int[] getDate()
{
	int[] toret = new int[ 3 ];
	Calendar now = Calendar.getInstance();

	toret[ 0 ] = now.get( Calendar.YEAR );
	toret[ 1 ] = now.get( Calendar.MONTH );
	toret[ 2 ] = now.get( Calendar.DAY_OF_MONTH );

	return toret;
}

/**
	@brief Returns the current date in the format "year-month-day",
           as in ISO 8601 format.
	@return Date as a human readable vector of char.
*/

public static char[] getStrDate()
{
	String toret = "";
	int[] date = getDate();

	toret += Integer.toString( date[ 0 ] );
	toret += '-';
	toret += Integer.toString( date[ 1 ] );
	toret += '-';
	toret += Integer.toString( date[ 2 ] );

	return toret.toCharArray();
}

/**
	@brief Returns the current time in the system
	@return Date as a vector of three ints: 0=hour,1=minute,2=second
*/

public static int[] getTime()
{
	int[] toret = new int[ 3 ];
	Calendar now = Calendar.getInstance();

	toret[ 0 ] = now.get( Calendar.HOUR_OF_DAY );
	toret[ 1 ] = now.get( Calendar.MINUTE );
	toret[ 2 ] = now.get( Calendar.SECOND );

	return toret;
}

/**
	@brief Returns the current time in the format "hour:minute:second"
	@return Date as a string
*/

public static char[] getStrTime()
{
	String toret = "";
	int[] time = getTime();

	toret += Integer.toString( time[ 0 ] );
	toret += ':';
	toret += Integer.toString( time[ 1 ] );
	toret += ':';
	toret += Integer.toString( time[ 2 ] );

	return toret.toCharArray();
}

/**
    @brief Returns the number of seconds from a given date to the beginning
           of the year of that date.
    @return The number of seconds, as an int.
 */
public static int getSecondsToDate(int year, int month, int day)
{
    Calendar date = Calendar.getInstance();

    date.set( year, month, day );
    return ( date.get( Calendar.DAY_OF_YEAR ) ) * SecondsInDay;
}

/**
     @brief Returns the number of seconds from the beginning
            of the year of the current date to that date
     @return The number of seconds, as an int.
 */
public static int getSecondsToNow()
{
    int toret;
    Calendar now = Calendar.getInstance();

    toret = now.get( Calendar.HOUR )
                * now.get( Calendar.MINUTE ) * now.get( Calendar.SECOND );
    toret += now.get( Calendar.DAY_OF_YEAR ) * SecondsInDay;

    return toret;
}

/**
	@brief Convert x from double to int
	@param x The value to be converted.
	@return The value finally converted, as an integer.
*/

public static int dblToInt(double x)
{
	return (int) x;
}

/**
	@brief Convert x from int to double
	@param x The value to be converted.
	@return The value finally converted, as a double.
*/

public static double intToDbl(int x)
{
	return (double) x;
}

/**
	@brief Convert x from char to int
	@param x The value to be converted.
	@return The value finally converted.
*/

public static int chrToInt(char x)
{
	return (int) x;
}

/**
	@brief Convert x from int to char
	@param x The value to be converted.
	@return The value finally converted.
*/

public static char intToChr(int x)
{
	return (char) x;
}

/**
	@brief Sleeps for ms miliseconds
	@param ms The number of ms to stop the program.
*/

public static void sleep(int ms)
{
    try {
	    Thread.sleep( ms );
    } catch(Exception ignored)
    {
    }

    return;
}


} // class util - jC standar library
