// Backend.java

/*
 * Backends: code generators
 */

package core;

import core.exceptions.SemanticError;
import core.intermediatecode.CodeUnit;

/**
 * Backends for the supported languages and frameworks.
 * The intermediate language (AST) gets translated into executable code.
 * @author baltasarq
 */
public abstract class Backend {

    public Backend(CodeUnit cdu)
    {
        this.cdu = cdu;
    }

    /**
     * Returns the code unit being translated into code.
     * @return The code unit, as a CodeUnit object.
     */
    public CodeUnit getCodeUnit()
    {
        return this.cdu;
    }

    /**
     * Visits the AST and generated the executable code.
     * @return The executable code, as a StringBuilder object.
     */
    public abstract byte[] generate() throws SemanticError;

    private CodeUnit cdu = null;
}
