// I18n.java

package core;

/**
 * Serves internationalization.
 * @author baltasarq
 */
public class I18n {
    /// The languages supported.
    public enum Language { EN, ES };
    public static String[] StrIsoLanguages = { "EN", "ES" };
    public static String[] StrEsLanguages = { "Inglés", "Español" };
    public static String[] StrEnLanguages = { "English", "Spanish" };
    public static String[][] StrLanguages = { StrEnLanguages, StrEsLanguages };

    
    /// The words supported.
    public enum Word {
        Expected,
        No,
        Not,
        Type,
        Parsing,
        Constant,
        Prohibited,
        Function,
        Variable,
        Or,
        Outside,
        Literal,
        Of,
        Instead,
        Missing,
        In,
        For,
        Duplicated,
        Block,
        Assign,
        Arguments,
        Call,
        Allowed,
        Unexpected,
        Declared,
        Array,
        Dimensions,
        Element,
        Number,
        Mode,
        Source,
        Prefix,
        Misplaced
    }
    
    /// The equivalent words in Spanish (ES_ES)
    public static final String[] StrESWord = {
        "se esperaba",
        "no",
        "no",
        "tipo",
        "analizando",
        "constante",
        "no está permitido",
        "función",
        "variable",
        "o",
        "fuera de",
        "literal",
        "de",
        "en cambio",
        "falta",
        "en",
        "para",
        "duplicado",
        "bloque",
        "asignar variable",
        "argumentos",
        "llamada a",
        "permitido",
        "inesperado",
        "declarado",
        "matriz",
        "dimensiones",
        "elemento",
        "num.",
        "modo",
        "fuente",
        "prefijo",
        "fuera de lugar",
    };
    
    public enum Sentence {
        TypeInference,
        DefaultLabelLastOne,
        ExpressionTerm,
        BadParsing,
        IsNotPrefixOperator,
        VariableDefOutBody,
        IllegalStartStatement,
        CodeOutBody,
        NonVoidType,
        MisplacedConstant,
        DoesNotMatchWith,
        SizeReferencesNull,
        SizeMustBeSpecified,
        ArrayAccess,
        DoesNotMatch,
        BreakOutLoop,
        ChrSizeOne,
        CntOutLoop,
        MustNotBeReserved,
        FirstCharInId,
        LengthExceededInId,
        AmbiguityError,
        SyntaxError,
        SemanticError,
        FileError,
        InvalidOpcode,
        MissingSemicolon,
        LoadingLibrary,
        IncorrectId,
        LanguageCode,
        MisplacedImport,
        InCallTo,
    }
    
    public static final String[] StrEnSentences = {
        "type inference",
        "default label should be the last one",
        "expression term",
        "bad parsing",
        "operator not supported as prefix operator",
        "variable definition outside function body",
        "illegal start of statement",
        "source code outside function body",
        "non-void type",
        "misplaced constant defnition",
        "does not match",
        "all size references must be null in array creation",
        "sizes for all dimensions of the array must be given",
        "array position access",
        "do not match",
        "break outside of loop",
        "char's must be only of length one",
        "continue outside loop",
        "must not be the reserved word",
        "first char should be letter or '_', in id",
        "maximum length exceeded in id",
        "ambiguity for",
        "syntax error",
        "semantic error",
        "file missing",
        "invalid opcode",
        "missing semicolon: ';'",
        "loading library",
        "incorrect id",
        "ISO 639-1 language code",
        "misplaced import",
        "in a call to",
    };
            
    public static final String[] StrEsSentences = {
        "inferencia de tipos",
        "la etiqueta default debe estar al final de todas",
        "componente de expr.",
        "error inesperado analizando fuente",
        "operador no soportado como operador prefijo",
        "variable definida fuera del cuerpo de fn.",
        "comienzo ilegal de sentencia",
        "instrucciones fuera del cuerpo de fn.",
        "tipo (aparte de void)",
        "def. de constante fuera de lugar",
        "no coincide con",
        "todas las referencias al num. de elementos deben ser null creando matriz",
        "deben indicarse todas las dimensiones de la matriz al ser creada",
        "acceso a un elemento de la matriz",
        "no coinciden",
        "break fuera de bucle",
        "los char's deben contener solo un elemento",
        "continue fuera de bucle",
        "no puede ser la palabra reservada",
        "primer char debe ser una letra o '_', en id",
        "longitud excesiva en id",
        "no es posible distinguir para el uso",
        "error de sintaxis",
        "error de uso",
        "archivo no encontrado",
        "opcode incorrecto",
        "falta punto y coma: ';'",
        "cargando lib",
        "id incorrecto",
        "id de lengua (ISO 639-1)",
        "import descolocado",
        "en la llamada a",
    };
    
    /**
     * Returns the translated (when needed) word.
     * @param w The word to translate
     * @return The word, in the current language, as String.
     */
    public static String get(Word w)
    {
        String toret;
        
        if ( language == Language.EN ) {
            toret = w.toString().toLowerCase();
        } else {
            toret = StrESWord[ w.ordinal() ];
        }
        
        return toret;
    }
    
    /**
     * Returns the translated sentenced.
     * @param s The sentences to translate.
     * @return The sentence translated, as a string.
     */
    public static String get(Sentence s)
    {
        String[] sentences = StrEnSentences;
        
        if ( language == Language.ES ) {
            sentences = StrEsSentences;
        }
        
        return sentences[ s.ordinal() ];
    }
    
    
    /// The current language being in use
    public static Language language = Language.ES;
}
