// Asg.java

package core.bshbackend;

import core.CompilingError;
import core.I18n;
import core.intermediatecode.Instr;
import core.intermediatecode.literals.ArrayLiteral;
import core.intermediatecode.RValue;
import core.intermediatecode.Type;
import core.intermediatecode.types.ArrayType;
import core.intermediatecode.types.NumericalType;
import core.exceptions.SemanticError;
import core.exceptions.semantic.IncorrectOpcode;

/**
 * Represents the opcode for the Asg mnemo
 * @author baltasarq
 */
public class Asg extends Opcode {

    public Asg(Instr instr) throws IncorrectOpcode
    {
        super( instr );
        this.rvalue = this.getInstr().getRValue();

        if ( this.rvalue instanceof Instr ) {
            this.rvalue = null;
            this.rvalueOp = Opcode.getOpcode( (Instr) this.getInstr().getRValue() );
        }
    }

    @Override
    public final core.intermediatecode.instrs.Asg getInstr()
    {
        return (core.intermediatecode.instrs.Asg) super.getInstr();
    }

    @Override
    public String emit() throws SemanticError
    {
        boolean isCastNeeded = false;
        StringBuilder toret = new StringBuilder();
        Type typeLvalue = null;
        Type typeRvalue = null;
                
        // Emit lvalue and '='
        toret.append( this.getInstr().getLValue().toString() );
        toret.append( Reserved.AssignOperator );
        
        // Chk if cast needed
        try {
            typeLvalue = this.getInstr().getType();
            typeRvalue = this.getInstr().getRValueType();
            
            if ( typeLvalue instanceof NumericalType ) {
                isCastNeeded = NumericalType.isAsgTypeShorter( typeLvalue, typeRvalue );
            }
        } catch(CompilingError exc)
        {
            throw new SemanticError(
                    Opcode.getPos(),
                    "jcg.ASG.emit(): " + I18n.get( I18n.Word.Missing )
                    + ' ' + I18n.get( I18n.Word.Type )
                    + ": '" + this.getInstr().toString() + '\''
            );
        }
       
        // Emit cast        
        if ( isCastNeeded ) {
            toret.append( '(' );
            toret.append( Reserved.getType( typeLvalue ) );
            toret.append( ')' );
        }

        // Emit rvalue
        if ( this.rvalue != null ) {
            // Chk if array presence needed
            if ( this.rvalue instanceof ArrayLiteral
              && !( (ArrayLiteral) this.rvalue ).isString() )
            {
                toret.append( "new " );
                toret.append( ( (ArrayType) typeRvalue ).cnvtTypeToString() );
            }
            
            // Add the rvalue itself
            toret.append( this.rvalue.toString() );
            
            if ( this.rvalue instanceof ArrayLiteral
              && ( (ArrayLiteral) this.rvalue ).isString() )
            {
                toret.append( ".toCharArray()" );
            }
        } else {
            toret.append( this.rvalueOp.emit() );
        }

        toret.append( Reserved.EndOfInstructionMark );
        return toret.toString();
    }

    private RValue rvalue = null;
    private Opcode rvalueOp = null;
}
