/*
 * Cal.java
 */

package core.bshbackend;

import core.exceptions.semantic.IncorrectOpcode;
import core.intermediatecode.Instr;
import core.intermediatecode.RefId;
import core.intermediatecode.Type;
import core.intermediatecode.instrs.Fun;
import core.intermediatecode.types.NumericalType;
import core.lexical.Position;

/**
 * Represnts the opcode generation for Call mnemos.
 * @author baltasarq
 */
public class Cal extends Opcode {

    public Cal(Instr instr)
    {
        super( instr );
    }

    @Override
    public core.intermediatecode.instrs.Cal getInstr()
    {
        return (core.intermediatecode.instrs.Cal) super.getInstr();
    }

    public Fun getLinkedFun() throws IncorrectOpcode
    {
        final Fun toret = this.getInstr().getLinkedFun();

        if ( toret == null ) {
            throw new IncorrectOpcode( new Position( "Cal.emit()", 0, 0 ), "linked function is not set" );
        }

        return toret;
    }

    public String emit() throws IncorrectOpcode
    {
        StringBuilder toret = new StringBuilder();
        final Type[] paramsTypes = this.getLinkedFun().getParamTypes();

        toret.append( this.getInstr().getFunctionId().toString() );
        toret.append( '(' );

        for(int i = 0; i < this.getInstr().getNumberOfArgs(); ++i) {
            final RefId argId = this.getInstr().getArg( i );
            final Type argType = paramsTypes[ i ];

            if ( argType instanceof NumericalType ) {
                toret.append( '(' + Reserved.getType( argType ) + ')' );
            }

            // Append the argument info and the comma
            toret.append( argId );

            if ( i < ( this.getInstr().getNumberOfArgs() -1 ) ) {
                toret.append( ',' );
            }
        }

        toret.append( ')' );
        toret.append( Reserved.EndOfInstructionMark );

        return toret.toString();
    }

}
