// Data.java

package core.intermediatecode;

import core.lexical.Id;
import core.lexical.Position;

/**
 * Represents Data in modules (definitions and constants)
 * @author baltasarq
 */
public abstract class Data extends Instr implements Member {

    public Data(Position pos)
    {
        super( pos );
        this.doc = new Documentation();
    }

    @Override
    public abstract Id getId();
    
    @Override
    public abstract Type getType();
    
    /**
     * Returns a printable description (as complete as needed) of the function
     * @return A description of this function, as a String object.
     */
    @Override
    public String getDesc()
    {
        String strModule = "";
        CodeUnit module = CodeBlock.getModuleOwner( this.getOwner() );
                
        if ( !( module instanceof Prg ) ) {
            strModule = module.getName() + ".";
        }
                
        return ( strModule + this.getId().getId() );
    }
    
    @Override
    public Documentation getDoc()
    {
        return this.doc;
    }

    public void setDoc(Documentation doc)
    {
        this.doc = doc;
    }
    
    @Override
    public String toHtml()
    {
        StringBuilder toret = new StringBuilder();
        String textComment = this.getDoc().getBrief();

        toret.append( this.getType().toHtml() );
        toret.append( " <b>" );
        toret.append( this.getId() );
        toret.append( "</b>" );

        if ( textComment.length() > 0 ) {
            toret.append( "\n<ul><li><i>" );
            toret.append( textComment );
            toret.append( "</i></ul>\n" );
        }

        return toret.toString();
    }

    protected Documentation doc;
}
