/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package core.intermediatecode;

import core.CompilingError;
import core.exceptions.SemanticError;
import core.intermediatecode.instrs.Fun;
import core.lexical.Position;

/**
 * Base class for all Instructions
 * @author baltasarq
 */
public abstract class Instr {

    /**
     * Creates a new instruction object
     * @param position The position this instr was created for.
     */
    public Instr(Position position)
    {
        this.position = position;
    }

    /**
     * Returns the position this instr was created for.
     * @return The position in the source code, a a core.Parser.Position object.
     */
    public Position getLineNumber()
    {
        return this.getPosition();
    }

    public void chk() throws CompilingError
    {
        Fun ownerFun = CodeBlock.getFunctionOwner( this.getOwner() );
        CodeUnit ownerModule = CodeBlock.getModuleOwner( this.getOwner() );
        
        if ( ownerFun == null
          && ownerModule == null )
        {
            throw new SemanticError( getPos(),
                    "jcg.Instr.chk(): Owner should be function or code unit" );
        }
    }

    /**
     * Returns the position of the instr in the source code
     * @return the position
     */
    public Position getPosition() {
        return position;
    }

    /**
     * Gets the owner of the instr
     * @return the code block owner
     * @see CodeBlockOwner
     */
    public CodeBlockOwner getOwner() {
        return this.cdo;
    }

    /**
     * Changes the owner of the instr
     * @param cdo the code block owner to set
     * @see CodeBlockOwner
     */
    public void setOwner(CodeBlockOwner cdu) {
        this.cdo = cdu;
    }

    /**
     * Returns a reference to the var (or constant) matching the Id.
     * @param id The name of the variable or constant.
     * @return A reference to the Instr object that defines it.
     */
    public Instr getData(String id)
    {
        Instr toret = null;

        if ( this.getOwner() instanceof Fun ) {
            Fun fun = (Fun) this.getOwner();

            toret = fun.lookForData( id );
        }
        else
        if ( this.getOwner() instanceof CodeUnit ) {
            CodeUnit cdu = (CodeUnit) this.getOwner();

            toret = cdu.lookForData( id );
        }

        return toret;
    }

    public static Position getPos() {
        return new Position( "AST (Instr)", 0, 0 );
    }

    @Override
    public abstract String toString();

    public abstract Type getType() throws CompilingError;

    private Position position;
    private CodeBlockOwner cdo = null;
}
