// Member.java

package core.intermediatecode;

import core.intermediatecode.visibilities.Visibility;
import core.lexical.Id;

/**
 * All members (constants and functions, maybe vars) in a module have a name.
 * @author baltasarq
 */
public interface Member {
    
    /**
     * Returns the id of this member (constant or function)
     * @return The id, as an Id object.
     */
    public Id getId();
    
    /**
     * Returns the type for the member
     * @return The type of the member, as a Type object.
     */
    public Type getType();
    
    /**
     * Returns the owner of the member
     * i.e. the module or program where it pertains to.
     * @return The owner of the member, as a CodeBlockOwner object.
     */
    public CodeBlockOwner getOwner();
    
    /**
     * Returns a description of this member.
     * @return A description of this member, suitable to be shown
     *         in an error report.
     */
    public String getDesc();
    
    /**
     * Returns the documentation for this member
     * @return The documentation associated with this member,
     *         as a Documentation object.
     */
    public Documentation getDoc();

    /**
     * Sets the documentation for this object.
     * @param doc The documentation to store for this object.
     */
    public void setDoc(Documentation doc);
    
    /**
     * Returns a representation of this member in HTML
     * @return The HTML representation, as a String.
     */
    public String toHtml();

    /**
     * Returns the visibility of the member
     * @return The visibility, as a Visibility object.
     */
    public Visibility getVisibility();

    /**
     * Decides whether the member is visible or not.
     * @return The visibility, as a boolean value: true if public, false otherwise.
     */
    public boolean isVisible();
}
