// AccAsg.java

package core.intermediatecode.instrs;

import core.CompilingError;
import core.I18n;
import core.exceptions.SemanticError;
import core.exceptions.semantic.AmbiguityError;
import core.exceptions.syntax.IncorrectId;
import core.intermediatecode.LValue;
import core.intermediatecode.RValue;
import core.intermediatecode.Type;
import core.lexical.Position;

/**
 * Represents assignments to array positions
 * @author baltasarq
 */
public class AccAsg extends Asg {
    public AccAsg(Position pos, Acc vPos, RValue rvalue) throws IncorrectId
    {
        super( pos, rvalue );

        this.vPos = vPos;
    }

    /**
     * Returns the acc (like "v[pos]") the rvalue is being assigned to.
     * @return The acc, as an Acc object
     */
    public Acc getVPos()
    {
        return this.vPos;
    }
    
    public LValue getLValue()
    {
        return this.getVPos();
    }

    @Override
    public String toString() {
        String toret = Name + " "
                     + this.getVPos().toString() + " <- "
                     + this.getRValue().toString()
        ;

        return toret;
    }

    @Override
    public void chk() throws CompilingError {
        super.chk();

        Def lvalue = null;
        Type lValueType = this.getType();

        this.getRValue().chk();
        Type rvalueType = this.getRValueType();
        
        if ( !Type.areCompatible( lValueType, rvalueType) ) {
            throw new SemanticError( this.getPosition(),
                    I18n.get( I18n.Word.Type )
                    + lValueType.toString()
                    + " & "
                    + rvalueType.toString()
                    + ' ' + I18n.get( I18n.Sentence.DoesNotMatch )
                    
            );
        }

        return;
    }

    public Type getType() throws IncorrectId, AmbiguityError, SemanticError, CompilingError
    {
        return this.getVPos().getType();
    }
    
    private Acc vPos;
}
