// Asg.java

package core.intermediatecode.instrs;

import core.intermediatecode.Data;
import core.Ast;
import core.CompilingError;
import core.I18n;
import core.exceptions.SemanticError;
import core.exceptions.syntax.IncorrectId;
import core.intermediatecode.Instr;
import core.intermediatecode.LValue;
import core.intermediatecode.RValue;
import core.intermediatecode.RefId;
import core.intermediatecode.Type;
import core.lexical.Position;

/**
 * This is the base class for all kinds of assignments
 * @author baltasarq
 */
public abstract class Asg extends Instr {
    public static final String Name = "ASG";
    
    public Asg(Position pos, RValue rvalue) throws IncorrectId
    {
        super( pos );
        
        this.rvalue = rvalue;
    }
    
    public RValue getRValue()
    {
        return this.rvalue;
    }
    
    public abstract LValue getLValue();
    
    public Type getRValueType() throws CompilingError, SemanticError, IncorrectId
    {
        RValue rvalueExpr = this.getRValue();
        Type toret = null;

        if ( rvalueExpr instanceof RefId ) {
            Data data = Ast.getAst().locateData( this.getPosition(), this.getOwner(), (RefId) rvalueExpr );
            
            if ( data == null ) {
                throw new SemanticError( this.getPosition(), I18n.get( I18n.Word.Missing )
                        + " rvalue: '"
                        + ( (RefId) rvalueExpr ).getId() + '\''
                );
            }
            
            toret = data.getType();
        } else {
            toret = rvalueExpr.getType();
        }
        
        return toret;
    }
    
    private RValue rvalue;
}
