// Cnt.java

package core.intermediatecode.instrs;

import core.intermediatecode.Type;
import core.CompilingError;
import core.exceptions.SemanticError;
import core.intermediatecode.CodeBlockOwner;
import core.intermediatecode.Instr;
import core.lexical.Position;
import core.I18n;

/**
 * Represents the continue instruction
 * @author baltasarq
 */
public class Cnt extends Instr {
    public static final String Name = "Cnt";

    public Cnt(Position pos)
    {
        super( pos );
    }
    
    @Override
    public String toString()
    {
        return Name;
    }

    @Override
    public Type getType() throws CompilingError
    {
        throw new CompilingError( this.getPosition(), "jcg.Cnt.getType(): "
                + Name + ' ' + I18n.get( I18n.Word.No )
                + ' ' + I18n.get( I18n.Word.Type )
        );
    }
    
    @Override
    public void chk() throws SemanticError
    {
        CodeBlockOwner cdo = this.getOwner();
        
        while ( cdo != null
             && !( cdo instanceof Whl ) )
        {
            cdo = cdo.getOwner();
        }
        
        if ( cdo == null ) {
            throw new SemanticError( this.getPosition(), I18n.get( I18n.Sentence.CntOutLoop ) );
        }

        return;
    }
    
}
