// Cra.java

package core.intermediatecode.instrs;

import core.intermediatecode.Type;
import core.intermediatecode.Instr;
import core.intermediatecode.RValue;
import core.intermediatecode.RefId;
import core.lexical.Position;

/**
 * Creates an array, given the type and number of elements
 * @author baltasarq
 */
public class Cra extends Instr implements RValue {
    public static final String Name = "CRA";

    public Cra(Position pos, Type type, RefId[] numElems)
    {
        super( pos );
        
        this.type = type;
        this.numElems = numElems;
    }
    
    @Override
    public String toString()
    {
        StringBuilder toret = new StringBuilder();
        
        toret.append( Name );
        toret.append( '(' );
        toret.append( this.getType().toString() );
        toret.append( ':' );
        for(RefId lim: this.getNumElems()) {
            toret.append( lim );
            toret.append( ' ' );
        }
        toret.append( ')' );
        
        return toret.toString();
    }
    
    /**
     * Returns the maximum number of elements for each dimension
     * @return The maximum number of elements for each dimension, as an int vector
     */
    public RefId[] getNumElems()
    {
        return this.numElems;
    }

    @Override
    public Type getType()
    {
        return this.type;
    }
    
    private Type type;
    private RefId[] numElems;
}
