// Def.java

package core.intermediatecode.instrs;

import core.I18n;
import core.exceptions.syntax.IncorrectId;
import core.intermediatecode.Data;
import core.intermediatecode.Type;
import core.intermediatecode.visibilities.PrivateVisibility;
import core.intermediatecode.visibilities.Visibility;
import core.lexical.Id;
import core.lexical.Position;
import core.lexical.Reserved;

/**
 *
 * @author baltasarq
 */
public class Def extends Data {
    public static final String Name = "DEF";

    public Def(Position pos, String id, Type type) throws IncorrectId
    {
        this( pos, new Id( id, pos ), type );
    }
    
    public Def(Position pos, Type type, Id id)
    {
        this( pos, id, type );
    }

    public Def(Position pos, Id id, Type type)
    {
        super( pos );
        
        this.id = id;
        this.type = type;
    }

    @Override
    public Type getType()
    {
        return this.type;
    }

    @Override
    public Id getId()
    {
        return this.id;
    }
    
    @Override
    public String toString()
    {
        String toret = Name + "["
                     + this.getType().toString()
                     + "] "
                     + this.getId().toString()
        ;

        return toret.toString();
    }

    @Override
    public void chk() throws IncorrectId
    {
        String lookId = ' ' + this.getId().getId() + ' ';
        int pos = Reserved.ReservedIdentifiers.indexOf( lookId );
                
        if ( pos >= 0 ) {
            throw new IncorrectId(
                    this.getPosition(),
                    "id " + I18n.get( I18n.Sentence.MustNotBeReserved )
                    + ": '" + this.getId().getId() + '\''
            );
        }
    }

    /**
     * This will be used when the variable was declared 'var', i.e. without type
     * @param type The new type (The old one should be AnyType).
     */
    public void setType(Type type) {
        this.type = type;
    }

    public Visibility getVisibility()
    {
        return PrivateVisibility.get();
    }

    public boolean isVisible()
    {
        return false;
    }

    private Id id;
    private Type type;
    
}
