// Fin.java

package core.intermediatecode.instrs;

import core.I18n;
import core.exceptions.syntax.IncorrectId;
import core.intermediatecode.Data;
import core.intermediatecode.Literal;
import core.intermediatecode.Type;
import core.intermediatecode.visibilities.PublicVisibility;
import core.intermediatecode.visibilities.Visibility;
import core.lexical.Id;
import core.lexical.Position;
import core.lexical.Reserved;

/**
 * Class representing constants
 * @author baltasarq
 */
public class Fin extends Data {
    public static final String Name = "FIN";

    public Fin(Position pos, Visibility visible, String id, Literal lit) throws IncorrectId
    {
        this( pos, visible, new Id( id, pos ), lit );
    }

    public Fin(Position pos, Id id, Literal lit)
    {
        this( pos, PublicVisibility.get(), id, lit );
    }

    public Fin(Position pos, String id, Literal lit) throws IncorrectId
    {
        this( pos, PublicVisibility.get(), new Id( id, pos ), lit );
    }

    public Fin(Position pos, Visibility visible, Id id, Literal lit)
    {
        super( pos );

        this.id = id;
        this.lit = lit;
        this.visible = visible;
    }

    @Override
    public Type getType()
    {
        return this.getLiteral().getType();
    }

    public Literal getLiteral()
    {
        return this.lit;
    }

    @Override
    public Id getId()
    {
        return this.id;
    }

    /**
     * Returns the visibility of the constant.
     * @return The visibility, as a Visibility object.
     */
    public Visibility getVisibility()
    {
        return this.visible;
    }

    /**
     * Decides whether the constant is visible or not.
     * @return The visibility, as a boolean value: true if public, false otherwise.
     */
    public boolean isVisible()
    {
        return this.visible.isVisible();
    }

    @Override
    public String toString()
    {
        String toret = Name
                     + " " + ( this.isVisible() ? '+' : '-' )
                     + " " + this.getId().toString()
                     + " " + this.getLiteral().toString()
        ;

        return toret;
    }

    @Override
    public void chk() throws IncorrectId
    {
        String lookId = ' ' + this.getId().getId() + ' ';
        int pos = Reserved.ReservedIdentifiers.indexOf( lookId );
                
        if ( pos >= 0 ) {
            throw new IncorrectId(
                    this.getPosition(),
                    "id " + I18n.get( I18n.Sentence.MustNotBeReserved )
                    + ": '" + this.getId().getId() + '\''
            );
        }
    }
    
    @Override
    public String toHtml()
    {
        StringBuilder toret = new StringBuilder();
        String textComment = this.getDoc().getBrief();

        toret.append( this.getVisibility().toHtml() );
        toret.append( " " );
        toret.append( "<b><font color=\"darkgreen\">final</font></b> " );
        toret.append( super.toHtml() );

        return toret.toString();
    }
    
    private Id id;
    private Literal lit;
    private Visibility visible;
}
