/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package core.intermediatecode.instrs;

import core.intermediatecode.Data;
import core.intermediatecode.Type;
import core.intermediatecode.DoubleCodeBlockOwner;
import core.intermediatecode.Instr;
import core.intermediatecode.CodeUnit;
import core.intermediatecode.RefId;
import core.intermediatecode.CodeBlock;
import core.Ast;
import core.CompilingError;
import core.exceptions.SemanticError;
import core.exceptions.syntax.IncorrectId;
import core.intermediatecode.types.BoolType;
import core.lexical.Position;
import core.I18n;

/**
 * Represents the if structure decision
 * @author baltasarq
 */
public class If extends Instr implements DoubleCodeBlockOwner
{
    public static final String Name = "IF";

    public If(Position pos, String id) throws IncorrectId
    {
        this( pos, new RefId( pos, id ) );
    }

    public If(Position pos, RefId id)
    {
        super( pos );
        
        this.var = id;
        this.instrsTrue = new CodeBlock( this );
        this.instrsFalse = new CodeBlock( this );
        this.currentCodeBlock = this.getFirstCodeBlock();
    }

    public RefId getVar()
    {
        return this.var;
    }

    @Override
    public String toString()
    {
        StringBuilder toret = new StringBuilder();
        
        toret.append( "if " );
        toret.append( this.getVar().toString() );
        toret.append( ":\n" );

        toret.append( this.getInstrsTrue().toString() );
        toret.append( " else: ");
        toret.append( this.getInstrsFalse().toString() );

        return toret.toString();
    }

    public CodeBlock getInstrsTrue()
    {
        return this.instrsTrue;
    }

    public CodeBlock getInstrsFalse()
    {
        return this.instrsFalse;
    }

    @Override
    public final CodeBlock getFirstCodeBlock()
    {
        return this.getInstrsTrue();
    }

    @Override
    public CodeBlock getSecondCodeBlock()
    {
        return this.getInstrsFalse();
    }

    @Override
    public CodeBlock getInstrs()
    {
        return this.getCurrentCodeBlock();
    }

    @Override
    public void chk() throws CompilingError
    {
        Data varDef = Ast.getAst().locateData( this.getPosition(), this.getOwner(), this.getVar() );

        if ( varDef == null ) {
            throw new SemanticError( this.getPosition(), I18n.get( I18n.Word.Missing )
                    + " variable '" + this.getVar().toString()
                    + '\''
            );
        }

        if ( varDef.getType() != BoolType.get() ) {
            throw new SemanticError( this.getPosition(), I18n.get( I18n.Word.Expected )
                    + ' ' + BoolType.Name
                    + ' ' + I18n.Word.Type
                  );
        }
        
        // Chk body 1
        for(Instr instr: this.getFirstCodeBlock()) {
            instr.chk();
        }
        
        // Chk body 2
        for(Instr instr: this.getSecondCodeBlock()) {
            instr.chk();
        }
    }

    @Override
    public Type getType() throws SemanticError {
        throw new SemanticError( this.getPosition(), "jcg.If.getType(): "
                + "if() " + I18n.get( I18n.Word.Type ) + '?'
        );
    }

    @Override
    public Data lookForData(String id) {
        return this.getOwner().lookForData( id );
    }

    @Override
    public Data lookForVar(String id) {
        return this.getOwner().lookForVar( id );
    }

    @Override
    public Data lookForConst(String id) {
        return this.getOwner().lookForConst( id );
    }
    
    @Override
    public Fun[] lookForFn(String id, Type[] paramTypes)
    {
        CodeUnit module = CodeBlock.getModuleOwner( this.getOwner() );
        
        return module.lookForFn( id, paramTypes );
    }
    
    public CodeBlock getCurrentCodeBlock()
    {
        return this.currentCodeBlock;
    }
    
    public void setCurrentCodeBlockToFirst()
    {
        this.currentCodeBlock = this.getFirstCodeBlock();
    }
    
    public void setCurrentCodeBlockToSecond()
    {
        this.currentCodeBlock = this.getSecondCodeBlock();
    }
    
    private RefId var;
    private CodeBlock instrsTrue = null;
    private CodeBlock instrsFalse = null;
    private CodeBlock currentCodeBlock = null;
}
