// Op.java

package core.intermediatecode.instrs;

import core.intermediatecode.Instr;
import core.intermediatecode.RValue;
import core.intermediatecode.RefId;
import core.lexical.Position;

/**
 * Represents operators
 * @author baltasarq
 */
public abstract class Op extends Instr implements RValue {

    public Op(Position pos, int numOps)
    {
        super( pos );
        this.ops = new RefId[ numOps ];
    }

    /**
     * Returns the number of operands (vars) involved
     * @return The operands, as an array of Id
     */
    public RefId[] getOps()
    {
        return this.ops;
    }

    /**
     * Return the number of operands involved.
     * @return That number, as an int.
     */
    public int getNumOps()
    {
        return this.ops.length;
    }

    @Override
    public String toString() {
        StringBuilder toret = new StringBuilder();

        for(int i = 0; i < this.getNumOps(); ++i) {
            toret.append( this.getOps()[ i ] );

            if ( i < ( this.getNumOps() -1 ) ) {
                toret.append( ',' );
            }
        }

        return toret.toString();
    }

    protected RefId[] ops;
}
