/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package core.intermediatecode.instrs;

import core.intermediatecode.Data;
import core.intermediatecode.Type;
import core.bshbackend.Reserved;
import core.CompilingError;
import core.exceptions.SemanticError;
import core.exceptions.syntax.IncorrectId;
import core.Ast;
import core.intermediatecode.CodeBlock;
import core.intermediatecode.Instr;
import core.intermediatecode.RefId;
import core.lexical.Position;
import core.I18n;

/**
 * Marks the returning value of a function.
 * @author baltasarq
 */
public class Ret extends Instr {
    public static String Name = "RET";

    public Ret(Position pos, String id) throws IncorrectId
    {
        this( pos, new RefId( pos, id ) );
    }

    public Ret(Position pos, RefId id)
    {
        super( pos );
        this.retval = id;
    }

    public RefId getId()
    {
        return this.retval;
    }

    @Override
    public String toString()
    {
        return Name + ' ' + this.getId().toString();
    }


    @Override
    public void chk() throws CompilingError
    {
        Fun owner = CodeBlock.getFunctionOwner( this.getOwner() );
        
        if ( this.getId() != null ) {
            Type type = this.getType();

            // Check whether the types of the function and of the return are compatible
            if ( owner != null ) {
                if ( owner.getType() == null ) {
                    throw new SemanticError( this.getPosition(),
                            I18n.get( I18n.Word.Type )
                                    + ' ' + I18n.get( I18n.Word.Of )
                                    + ' ' + Reserved.Return
                                    + ' ' + I18n.get( I18n.Sentence.DoesNotMatch )
                                    + " void: " + owner.getDesc()
                    );
                }
                else
                if ( !owner.getType().equals( type )  ) {
                    throw new SemanticError( this.getPosition(),
                            I18n.get( I18n.Word.Type )
                            + ' ' + I18n.get( I18n.Word.Of )
                            + ' ' + Reserved.Return
                            + ' ' + I18n.get( I18n.Sentence.DoesNotMatch )
                            + " fn.: " + owner.getDesc()
                    );
                }
            } else {
                throw new SemanticError( this.getPosition(), I18n.get( I18n.Word.Unexpected ) + ' ' + Reserved.Return );
            }
        } else {
            if ( owner.getType() != null ) {
                throw new SemanticError( this.getPosition(),
                        I18n.get( I18n.Word.Missing )
                        + ' ' + Reserved.Return
                        + ' ' + I18n.get( I18n.Word.For )
                        + " fn.: " + owner.getDesc()
                );
            }
        }
    }

    @Override
    public Type getType() throws CompilingError
    {
        Type toret = null;

        if ( this.getId() != null ) {
            String id = this.getId().getId();
            Data var = Ast.getAst().locateData( this.getPosition(), this.getOwner(), this.getId() );

            if ( var == null ) {
                throw new SemanticError( this.getPosition(), I18n.get( I18n.Word.Missing )
                        + " variable: " + this.getId()
                        + ' ' + I18n.get( I18n.Word.Of )
                        + ' ' + Reserved.Return
                );
            }

            toret = var.getType();
        }
        
        return toret;
    }
    
    private RefId retval;
}
