// VarAsg.java

package core.intermediatecode.instrs;

import core.intermediatecode.Data;
import core.Ast;
import core.CompilingError;
import core.I18n;
import core.exceptions.SemanticError;
import core.exceptions.semantic.AmbiguityError;
import core.exceptions.syntax.IncorrectId;
import core.intermediatecode.LValue;
import core.intermediatecode.RValue;
import core.intermediatecode.RefId;
import core.intermediatecode.Type;
import core.intermediatecode.types.AnyType;
import core.lexical.Position;

/**
 * Represents an assignment to a variable.
 * @author baltasarq
 */
public class VarAsg extends Asg {
    

    public VarAsg(Position pos, String id, RValue rvalue) throws IncorrectId
    {
        this( pos, new RefId( pos, id ), rvalue );
    }

    public VarAsg(Position pos, String id, String id2) throws IncorrectId
    {
        this( pos, new RefId( pos, id ), new RefId( pos, id2 ) );
    }

    public VarAsg(Position pos, RefId id, RValue rvalue) throws IncorrectId
    {
        super( pos, rvalue );

        this.id = id;
    }

    /**
     * Returns the refid the rvalue is being assigned to.
     * @return The id, as a RefId object
     */
    public RefId getId()
    {
        return this.id;
    }
    
    @Override
    public LValue getLValue()
    {
        return this.getId();
    }

    @Override
    public String toString() {
        String toret = Name + " "
                     + this.getId().toString() + " <- "
                     + this.getRValue().toString()
        ;

        return toret;
    }

    @Override
    public void chk() throws CompilingError {
        super.chk();

        Def lvalue = null;
        Data var = Ast.getAst().locateData( this.getPosition(), this.getOwner(), this.getId() );

        if ( var == null ) {
            throw new SemanticError( this.getPosition(), I18n.get( I18n.Word.Missing )
                    + " variable: '" + this.getId() + '\''
            );
        }
        else
        // TODO: Fix this!! Does not make any fucking sense!!
        if ( var instanceof Fin )
        {
            throw new SemanticError( this.getPosition(), I18n.get( I18n.Word.Missing )
                    + " final: '" + this.getId() + '\''
            );
        }

        lvalue = (Def) var;
        Type varType = lvalue.getType();
        this.getRValue().chk();
        Type rvalueType = this.getRValueType();
        
        if ( !Type.areCompatible( varType, rvalueType) ) {
            throw new SemanticError( this.getPosition(), I18n.get( I18n.Word.Type )
                    + ' ' + I18n.get( I18n.Word.Of )
                    + " variable: '" + varType.toString() + '\''
                    + ' ' + I18n.get( I18n.Sentence.DoesNotMatchWith )
                    + ' ' + I18n.get( I18n.Word.Type )
                    + ' ' + I18n.get( I18n.Word.Of )
                    + " rvalue: '" + rvalueType.toString() + '\''                    
            );
        }
        
        // Adjust the type -- this is needed to avoid anys to be transferred
        // to the generated code.
        if ( varType == AnyType.get() ) {
            lvalue.setType( rvalueType );
        }
        
        return;
    }

    @Override
    public Type getType() throws IncorrectId, AmbiguityError
    {
        Data data = Ast.getAst().locateData( this.getPosition(), this.getOwner(), this.getId() );

        return data.getType();
    }
    
    private RefId id;
}
