/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package core.intermediatecode.instrs;

import core.intermediatecode.Data;
import core.intermediatecode.Type;
import core.intermediatecode.Instr;
import core.intermediatecode.CodeBlockOwner;
import core.intermediatecode.CodeUnit;
import core.intermediatecode.RefId;
import core.intermediatecode.CodeBlock;
import core.Ast;
import core.CompilingError;
import core.exceptions.SemanticError;
import core.exceptions.syntax.IncorrectId;
import core.intermediatecode.types.BoolType;
import core.lexical.Position;
import core.I18n;

/**
 * Represents the while decision structure
 * @author baltasarq
 */
public class Whl extends Instr implements CodeBlockOwner {
    public static final String Name = "WHL";

    public Whl(Position pos, String id) throws IncorrectId
    {
         this( pos, new RefId( pos, id ) );
    }

    public Whl(Position pos, RefId id)
    {
        super( pos );
        this.var = id;
        this.cdb = new CodeBlock( this );
    }

    public RefId getVar()
    {
        return this.var;
    }

    @Override
    public CodeBlock getInstrs()
    {
        return this.cdb;
    }

    @Override
    public String toString()
    {
        StringBuilder toret = new StringBuilder();

        toret.append( Name  );
        toret.append( ' ' );
        toret.append( this.getVar().toString() );
        toret.append( this.getInstrs().toString() );

        return toret.toString();
    }

    @Override
    public void chk() throws CompilingError
    {
        Data controlVar = Ast.getAst().locateData( this.getPosition(), this.getOwner(), this.getVar() );

        if ( controlVar == null ) {
            throw new SemanticError( this.getPosition(), I18n.get( I18n.Word.Missing )
                    + " variable: '" + this.getVar().getId() + '\''
            );
        }

        if ( controlVar.getType() != BoolType.get() ) {
            throw new SemanticError( this.getPosition(), I18n.get( I18n.Word.Expected )
                    + ' ' + BoolType.Name
            );
        }
        
        // Chk body
        for(Instr instr: this.getInstrs()) {
            instr.chk();
        }
    }

    @Override
    public Type getType() throws SemanticError {
        Data controlVar = this.getOwner().lookForData( this.getVar().getId() );

        if ( controlVar == null ) {
            throw new SemanticError( this.getPosition(), I18n.get( I18n.Word.Missing )
                    + " variable: '" + this.getVar().getId() + '\''
            );
        }

        return controlVar.getType();
    }

    @Override
    public Data lookForData(String id) {
        return this.getOwner().lookForData( id );
    }

    @Override
    public Data lookForVar(String id) {
        return this.getOwner().lookForVar( id );
    }

    @Override
    public Data lookForConst(String id) {
        return this.getOwner().lookForConst( id );
    }
    
    @Override
    public Fun[] lookForFn(String id, Type[] paramTypes)
    {
        CodeUnit module = CodeBlock.getModuleOwner( this.getOwner() );
        
        return module.lookForFn( id, paramTypes );
    }
    
    private RefId var = null;
    private CodeBlock cdb = null;
}
