// BoolLiteral.java

package core.intermediatecode.literals;

import core.exceptions.SemanticError;
import core.exceptions.SyntaxError;
import core.I18n;
import core.intermediatecode.Literal;
import core.intermediatecode.types.BoolType;
import core.lexical.Reserved;

/**
 * Represents literals for booleans
 * @author baltasarq
 */
public class BoolLiteral extends Literal {

    public BoolLiteral(boolean value)
    {
        this.value = value;
    }

    @Override
    public Boolean getLiteral()
    {
        return this.value;
    }

    @Override
    public void setLiteral(String lit)
    {
        this.value = Boolean.parseBoolean( lit );
    }

    @Override
    public void setLiteral(Object lit) throws SemanticError
    {
        if ( lit.getClass() != Boolean.class ) {
            throw new SemanticError( getPos(), "jcg.Boolean,setLiteral(): "
                    + I18n.get( I18n.Word.Expected )
                    + ' ' + BoolType.Name
            );
        }

        this.value = (Boolean) lit;
    }

    @Override
    public BoolType getType() {
        return BoolType.get();
    }

    @Override
    public String toString()
    {
        return Boolean.toString( this.getLiteral() );
    }
    
    public static BoolLiteral create(String lit) throws SyntaxError
    {
        BoolLiteral toret = null;
        lit = lit.trim();
        
        if ( lit.equals( Reserved.True ) ) {
            toret = new BoolLiteral( true );
        }
        else
        if ( lit.equals( Reserved.False ) ) {
            toret = new BoolLiteral( false );
        }
        else {
            throw new SyntaxError( Literal.getPos(), I18n.get( I18n.Word.Expected )
                    + " literal " + I18n.get( I18n.Word.Of )
                    + ' ' + I18n.get( I18n.Word.Type )
                    + ' ' + BoolType.Name
            );
        }
        
        return toret;
    }

    protected boolean value = false;
}
