// CharLiteral.java

package core.intermediatecode.literals;

import core.CompilingError;
import core.exceptions.SemanticError;
import core.I18n;
import core.intermediatecode.Literal;
import core.intermediatecode.types.CharType;
import core.lexical.Position;

/**
 *
 * @author baltasarq
 */
public class CharLiteral extends Literal {

    public CharLiteral(char lit)
    {
        this.lit = lit;
    }

    @Override
    public void setLiteral(String lit) {
        this.lit = lit.charAt( 0 );
    }

    @Override
    public void setLiteral(Object lit) throws SemanticError
    {
        if (  lit.getClass() != Character.class ) {
            throw new SemanticError( getPos(), "jcg.CharLiteral.setLiteral(): "
                    + I18n.get( I18n.Word.Expected )
                    + ' ' + CharType.Name
            );
        }

        this.lit = (Character) lit;
    }

    @Override
    public CharType getType() {
        return CharType.get();
    }

    @Override
    public Character getLiteral()
    {
        return this.lit;
    }

    @Override
    public String toString()
    {
        return "'" + Character.toString( this.lit ) + "'";
    }

    public static CharLiteral parse(String value) throws CompilingError
    {
        CharLiteral toret = null;

        try {
            toret = new CharLiteral( value.charAt( 0 ) );


            if ( value.length() > 1 ) {
                throw new SemanticError (
                    new Position( "jcg.CharLiteral.parse", 0, 0 ),
                    I18n.get( I18n.Sentence.ChrSizeOne )
                );
            }
        } catch(CompilingError exc) {
            throw exc;
        } catch(Exception exc) {
            throw new SemanticError (
                new Position( "jcg.CharLiteral.parse", 0, 0 ),
                exc.getMessage()
            );
        }

        return toret;
    }

    public static String parseChar(char x)
    {
        String toret = Character.toString( x );

        if ( x == '\\' ) {
            lastChar = x;
            toret = "";
        } else {
            if ( lastChar == '\\' ) {
                if ( x == 'n' ) {
                    toret = "\n";
                }
                else
                if ( x == 't' ) {
                    toret = "\t";
                }
                else
                if ( x == '\'' ) {
                    toret = "\'";
                }
                else
                if ( x == '"' ) {
                    toret = "\"";
                }
                else
                if ( x == '\\' ) {
                    toret = "\\";
                }
            }

            lastChar = '\0';
        }

        return toret;
    }

    private static char lastChar;
    private char lit;
}
