// DoubleLiteral.java

package core.intermediatecode.literals;

import core.exceptions.SemanticError;
import core.I18n;
import core.intermediatecode.types.DoubleType;

/**
 * Class representing literals for real numbers.
 * @author baltasarq
 */
public class DoubleLiteral extends NumericalLiteral {
    private double lit;

    public DoubleLiteral(double lit)
    {
        this.lit = lit;
    }

    @Override
    public String toString()
    {
        return Double.toString( this.lit );
    }

    @Override
    public void setLiteral(String lit) {
        this.lit = Double.parseDouble( lit );
    }

    @Override
    public void setLiteral(Object lit) throws SemanticError
    {
        if (  lit.getClass() != Double.class ) {
            throw new SemanticError( getPos(), "jcg.DoubleLiteral,setLiteral(): "
                    + I18n.get( I18n.Word.Expected )
                    + ' ' + DoubleType.Name
            );
        }

        this.lit = (Double) lit;
    }

    @Override
    public Double getLiteral()
    {
        return this.lit;
    }

    @Override
    public DoubleType getType()
    {
        return DoubleType.get();
    }
}
