// StringLiteral.java

package core.intermediatecode.literals;

import core.exceptions.SemanticError;
import core.I18n;
import core.intermediatecode.types.ArrayType;
import core.intermediatecode.types.CharType;

/**
 *
 * @author baltasarq
 */
public class StringLiteral extends ArrayLiteral {

    public StringLiteral(String txt) throws SemanticError
    {
        super( ArrayType.get( CharType.get(), 1 ), txt.toCharArray() );
    }

    @Override
    public void setLiteral(String lit) throws SemanticError
    {
        super.setLiteral( lit.toCharArray() );
    }

    @Override
    public void setLiteral(Object lit) throws SemanticError
    {
        if ( lit instanceof String ) {
            super.setLiteral( ( (String) lit ).toCharArray() );
        } else {
            if (  !lit.getClass().isArray()
               || lit.getClass().getComponentType() != char.class )
            {
                throw new SemanticError( getPos(),
                        "jcg.StringLiteral.setLiteral(): "
                        + I18n.get( I18n.Word.Expected )
                        + " char[]"
                );
            }

            super.setLiteral( lit );
        }
    }

    @Override
    public String getLiteral()
    {
        return String.valueOf( (char[]) super.getLiteral() );
    }

    @Override
    public String toString()
    {
        return "\"" + this.getLiteral() + "\"";
    }

}
