// ArrayType.java

package core.intermediatecode.types;

import core.intermediatecode.Type;

/**
 * Represents the array type.
 * @author baltasarq
 */
public class ArrayType extends Type {
    public static final String Name = "[]";

    public ArrayType(Type type, int dimensions)
    {
        this.dimensions = dimensions;
        this.type = type;
    }

    public static ArrayType get(Type type, int dimensions)
    {
        ArrayType toret = null;

        if ( type instanceof StringType
          && dimensions == 1 )
        {
            toret = StringType.get();
        } else {
            toret = new ArrayType( type, dimensions );
        }

        return toret;
    }

    /**
     * Returns the number of dimensions in the array
     * @return The number of dimensions, as an int.
     */
    public int getDimensions()
    {
        return this.dimensions;
    }

     /**
     * Returns the number of dimensions in the array
     * @return The number of dimensions, as an int.
     */
    public String getDimensionsAsString()
    {
        StringBuilder toret = new StringBuilder();

        for(int i = 0; i < this.getDimensions(); ++i) {
            toret.append( Name );
        }

        return toret.toString();
    }

    /**
     * Returns the type of the array
     * @return The array, as a Type object.
     */
    public Type getType()
    {
        return this.type;
    }

    @Override
    public String toString()
    {
        StringBuilder toret = new StringBuilder();

        toret.append( this.getType().toString() );
        toret.append( this.getDimensionsAsString() );

        return toret.toString();
    }

    private int dimensions = 1;
    private Type type;

    @Override
    public Class getJavaType() {
        return java.lang.reflect.Array.newInstance(
                        this.getType().getJavaType(),
                        dimensions ).getClass()
        ;
    }
    
    public String cnvtTypeToString()
    {
        StringBuilder toret = new StringBuilder();
        
        toret.append( this.getType().getJavaType().toString() );
        toret.append( this.getDimensionsAsString() );
        
        return toret.toString();
    }

    public boolean isString()
    {
        return ( this.getType() == CharType.get()
              && this.getDimensions() == 1 );
    }
    
    public boolean isCompatible(Object obj)
    {
        boolean toret = false;
        ArrayType array = null;
        
        if ( obj instanceof ArrayType ) {
            array = (ArrayType) obj;
            
            toret = ( Type.areCompatible( this.getType(), array.getType() )
                   && this.getDimensions() == array.getDimensions() );
        }
        
        return toret;
    }
    
    @Override
    public boolean equals(Object obj)
    {
        boolean toret = false;
        ArrayType array = null;
        
        if ( obj instanceof ArrayType ) {
            array = (ArrayType) obj;
            
            toret = ( this.getType() == array.getType()
                   && this.getDimensions() == array.getDimensions() );
        }
        
        return toret;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 89 * hash + this.dimensions;
        hash = 89 * hash + (this.type != null ? this.type.hashCode() : 0);
        return hash;
    }
}
