// NumericalType.java

package core.intermediatecode.types;

import core.intermediatecode.Type;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Represents all numerical types: char, int, and double
 * @author baltasarq
 */
public abstract class NumericalType extends Type {

    public static final Class[] NumericalTypeWideness = { 
        CharType.class,
        IntType.class,
        DoubleType.class
    };
    
    /**
     * Determines whether the rvalue type is shorter than the lvalue's one
     * @param asigner The type of the lvalue
     * @param asignee The type of the rvalue
     * @return True if the type of the lvalue is shorter than the one of the lvalue
     */
    public static boolean isAsgTypeShorter(Type asigner, Type asignee)
    {
        boolean toret = false;
        int posAsigner = -1;
        int posAsignee = -1;
        
        // Look for the type
        for(int i = 0; i < NumericalTypeWideness.length; ++i) {
            try {
                Type type = (Type) NumericalTypeWideness[ i ].getDeclaredMethod( "get" ).invoke( null );
                
                if ( type == asigner )
                {
                    posAsigner = i;
                }

                if ( type == asignee )
                {
                    posAsignee = i;
                }
            } catch (Exception ex) {
                toret = true;
            }
        }
        
        // Both should be numerical at this point, but...
        if ( posAsigner == -1
          || posAsignee == -1 )
        {
            toret = true;
        } else {
            toret = ( posAsignee > posAsigner );
        }        
        
        return toret;
    }

    /**
     * Given two types, determines the superset of both of them
     * @param type1 The first type
     * @param type2 The second type
     * @return A type that is able to represent both of them
     */
    public static Type getWidestOfTypes(Type type1, Type type2)
    {
        Type toret = type1;
        int posType1 = -1;
        int posType2 = -1;
        
        // Adjust types
        if ( toret == CharType.get() ) {
            toret = IntType.get();
        }
        
        if ( type2 == CharType.get() ) {
            type2 = IntType.get();
        }
        
        // Look for the type
        for(int i = 0; i < NumericalTypeWideness.length; ++i) {
            try {
                Type type = (Type) NumericalTypeWideness[ i ].getDeclaredMethod( "get" ).invoke( null );
                
                if ( type == toret )
                {
                    posType1 = i;
                }

                if ( type == type2 )
                {
                    posType2 = i;
                }
            } catch (Exception ex) {
                
            }
        }
        
        // Adjust types for the widest
        if ( posType1 != -1
          && posType2 != -1 )
        {
            if ( posType2 > posType1 ) {
                toret = type2;
            }
        }
        
        return toret;
    }
}
