// Visibility.java

package core.intermediatecode.visibilities;

import core.lexical.Reserved;

/**
 * Represents visibility for entities in a program.
 * @author baltasarq
 */
public abstract class Visibility {
    public static final String PublicIds = " pub public ";
    public static final String PrivateIds = " prv private ";

    @Override
    public abstract String toString();

    public boolean isVisible()
    {
        return ( this instanceof PublicVisibility );
    }

    /**
     * Returns the appropriate visibility object from the identifier
     * @param id A String with the id found
     * @return null if it is not a visibility id, the appropriate object otherwise
     */
    public static Visibility createVisibility(String id)
    {
        Visibility toret = null;

        if ( PublicIds.indexOf( id ) >= 0 ) {
            toret = PublicVisibility.get();
        }
        else
        if ( PrivateIds.indexOf( id ) >= 0 ) {
            toret = PrivateVisibility.get();
        }

        return toret;
    }
    
    /**
     * Returns the HTML representation
     * @return HTML in a string
     */
    public String toHtml()
    {
        StringBuffer toret = new StringBuffer();

        toret.append( "<b><font color=\"navy\">" );

        if ( this.isVisible() ) {
            toret.append( Reserved.Public );
        } else {
            toret.append( Reserved.Private );
        }

        toret.append( "</font></b>" );

        return toret.toString();
    }
}
