// Asg.java

package core.jsbackend;

import core.exceptions.SemanticError;
import core.exceptions.semantic.IncorrectOpcode;
import core.intermediatecode.Instr;
import core.intermediatecode.RValue;
import core.intermediatecode.literals.ArrayLiteral;

/**
 * Represents the opcode for the Asg mnemo
 * @author baltasarq
 */
public class Asg extends Opcode {

    public Asg(Instr instr) throws IncorrectOpcode
    {
        super( instr );
        this.rvalue = this.getInstr().getRValue();

        if ( this.rvalue instanceof Instr ) {
            this.rvalue = null;
            this.rvalueOp = Opcode.getOpcode((Instr) this.getInstr().getRValue());
        }
    }

    @Override
    public final core.intermediatecode.instrs.Asg getInstr()
    {
        return (core.intermediatecode.instrs.Asg) super.getInstr();
    }

    @Override
    public String emit() throws SemanticError
    {
        StringBuilder toret = new StringBuilder();

        // Emit lvalue and '='
        toret.append( this.getInstr().getLValue().toString() );
        toret.append( Reserved.AssignOperator );

        // Emit rvalue
        if ( this.rvalue != null ) {
/*            // Chk if array presence needed
            if ( this.rvalue instanceof ArrayLiteral
              && !( (ArrayLiteral) this.rvalue ).isString() )
            {
                toret.append( Reserved.NewOperator );
                toret.append( ' ' );
                toret.append( Reserved.ArrayMark );
                toret.append( '(' );
                toret.append( ( (ArrayType) typeRvalue ).cnvtTypeToString() );
                toret.append( ')' );
            }
  */
            // Add the rvalue itself
            toret.append( this.rvalue.toString() );

            if ( this.rvalue instanceof ArrayLiteral
              && ( (ArrayLiteral) this.rvalue ).isString() )
            {
                toret.append( ".split( '' )" );
            }
        } else {
            toret.append( this.rvalueOp.emit() );
        }

        toret.append( Reserved.EndOfInstructionMark );
        return toret.toString();
    }

    private RValue rvalue = null;
    private Opcode rvalueOp = null;
}
