// Opcode.java

package core.jsbackend;

import core.I18n;
import core.exceptions.SemanticError;
import core.exceptions.semantic.IncorrectOpcode;
import core.intermediatecode.CodeBlock;
import core.intermediatecode.Instr;
import core.lexical.Position;

/**
 *
 * @author baltasarq
 */
public abstract class Opcode {

    public Opcode(Instr instr)
    {
        this.instr = instr;
    }

    /**
     * Generates the source code corresponding to the opcode
     * @return The source code, as a string.
     */
    public abstract String emit() throws SemanticError;

    /**
     * Returns the associated mnemo
     * @return The associated mnemo, as an Instr object.
     */
    public Instr getInstr()
    {
        return this.instr;
    }

    public static Opcode getOpcode(Instr instr) throws IncorrectOpcode
    {
        Opcode toret = null;

        if ( instr instanceof core.intermediatecode.instrs.Asg ) {
            toret = new Asg( instr );
        }
        else
        if ( instr instanceof core.intermediatecode.instrs.Cal ) {
            toret = new Cal( instr );
        }
        else
        if ( instr instanceof core.intermediatecode.instrs.BinaryOp ) {
            toret = new BinaryOp( instr );
        }
        else
        if ( instr instanceof core.intermediatecode.instrs.Def ) {
            toret = new Def( instr );
        }
        else
        if ( instr instanceof core.intermediatecode.instrs.If ) {
            toret = new If( instr );
        }
        else
        if ( instr instanceof core.intermediatecode.instrs.Whl ) {
            toret = new Whl( instr );
        }
        else
        if ( instr instanceof core.intermediatecode.instrs.Fin ) {
            toret = new Fin( instr );
        }
        else
        if ( instr instanceof core.intermediatecode.instrs.Ret ) {
            toret = new Ret( instr );
        }
        else
        if ( instr instanceof core.intermediatecode.instrs.Not ) {
            toret = new Not( instr );
        }
        else
        if ( instr instanceof core.intermediatecode.instrs.Acc ) {
            toret = new Acc( instr );
        }
        else
        if ( instr instanceof core.intermediatecode.instrs.Brk ) {
            toret = new Brk( instr );
        }
        else
        if ( instr instanceof core.intermediatecode.instrs.Cnt ) {
            toret = new Cnt( instr );
        }
        else
        if ( instr instanceof core.intermediatecode.instrs.Cra ) {
            toret = new Cra( instr );
        }
        else
        if ( instr instanceof core.intermediatecode.instrs.Nop ) {
            toret = new Nop( instr );
        }
        else throw new IncorrectOpcode( getPos(),
                "jcg.Opcode.getOpcode(): " + I18n.get( I18n.Sentence.InvalidOpcode )
        );

        return toret;
    }

    public static Position getPos() {
        return new Position( "AST (opcode)", 0, 0 );
    }

    public static void generateBody(CodeBlock cb, StringBuilder builder) throws SemanticError
    {
        builder.append( '{' );
        builder.append( '\n' );

        for(int i = 0; i < cb.size(); ++i) {
            builder.append( Opcode.getOpcode( cb.get( i ) ).emit() );
            builder.append( '\n' );
        }

        builder.append( '}' );
    }

    private Instr instr;
}
