// Id.java

package core.lexical;

import core.I18n;
import core.intermediatecode.types.CharType;
import core.exceptions.syntax.IncorrectId;

/**
 * Class representing Id's
 * Id's denote the name of a function, a variable or a constant.
 * @author baltasarq
 */
public class Id {
    public static final int MaxIdLength = 255;
    public static final String TmpId = "____tmp_jc";

    public static Id createTemp() throws IncorrectId {
        String toret = "";
        
        // Crear el id
        ++numIds;
        toret = TmpId + Integer.toString( numIds );
        
        return new Id( toret, getFakePos() );
    }

    public Id(String strId, Position pos) throws IncorrectId
    {
        this.pos = pos;
        this.setId( strId );
    }

    public String getId()
    {
        return this.id;
    }
    
    public Position getPos()
    {
        return this.pos;
    }
    
    public final void setId(String strId) throws IncorrectId
    {
        strId = strId.trim();
        
        this.id = strId;
        chk();
    }
    
    public static Position getFakePos()
    {
        return ( new Position( "Id.chk", 0, 0 ) );
    }
    
    /**
     * Calls chkId() in order to verify the correctness of the id,
     * @throws IncorrectId 
     */
    public void chk() throws IncorrectId
    {
        chkId( this.getId(), this.getPos() );
    }

    
    /**
     * Checks that a given Is, in string form, is correct.
     * This is to be used internally
     * @param strId A string containing the strId-to-be.
     * @throws IncorrectId
     */
    public static void chkId(String strId) throws IncorrectId
    {
        chkId( strId, null );
    }
    
    /**
     * Checks that a given Is, in string form, is correct.
     * @param strId A string containing the strId-to-be.
     * @throws IncorrectId
     */
    public static void chkId(String strId, Position pos) throws IncorrectId
    {
        // Fix pos, if needed
        if ( pos == null ) {
            pos = new Position( "gcj.Id.chkId()", 0, 0 );
        }
        
        // First letter
        char ch = strId.charAt( 0 );
        
        if ( ch != '_'
          && !Character.isLetter( ch ) )
        {
            throw new IncorrectId( pos, I18n.get( I18n.Sentence.FirstCharInId )
                    + ": " + strId
            );
        }

        // Size
        if ( strId.length() >  MaxIdLength ) {
            throw new IncorrectId( pos, I18n.get( I18n.Sentence.LengthExceededInId )
                    + ": '" + strId + '\''
            );
        }

        // Check other characters in the strId's string
        for(int i = 1; i < strId.length(); ++i) {
            ch = strId.charAt( i );
            
            if ( ch != '_'
              && ch != '.'
              && !Character.isLetter( ch )
              && !Character.isDigit( ch ) )
            {
                throw new IncorrectId( pos, CharType.Name
                        + ' ' + I18n.get( I18n.Word.Prohibited )
                        + ": '" + ch
                        + "', id: '" + strId + '\''
                );
            }
        }
        
        return;
    }

    @Override
    public String toString()
    {
        return this.getId();
    }

    protected static int numIds = 0;
    private Position pos;
    private String id;
}
