// Position.java

package core.lexical;

/**
 * Represents an position inside a line of the source code.
 * @author baltasarq
 */
public class Position {
    /**
     * Creates a new position object, referring to the source code.
     * @param x The line number
     * @param y The character inside the line.
     */
    public Position(String fileName, long x, int y)
    {
        this.fileName = fileName;
        this.numLine = x;
        this.numChar = y;
    }

    /**
     * Returns the line number
     * @return The line number as an int
     */
    public long getNumLine()
    {
        return this.numLine;
    }

    /**
     * Returns the character number, inside the line.
     * @return The character number inside the line as an int.
     */
    public int getNumCharacter()
    {
        return this.numChar;
    }

    /**
     * Returns the file name in which this position is valid
     * @return The file name, as a String
     */
    public String getFileName()
    {
        return this.fileName;
    }

    private long numLine;
    private int numChar;
    private final String fileName;
}
