// Reserved.java

package core.lexical;

import core.intermediatecode.Type;
import core.intermediatecode.types.AnyType;
import core.intermediatecode.types.BoolType;
import core.intermediatecode.types.CharType;
import core.intermediatecode.types.DoubleType;
import core.intermediatecode.types.IntType;

/**
 *
 * @author baltasarq
 */
public class Reserved {
    // Marks
    public static final String OneLineCommentMark = "//";
    public static final String MultiLineCommentMark = "/*";
    public static final String MultiLineDocCommentMark = "/**";
    public static final String EndMultiLineCommentMark = "*/";
    public static final String EndOfInstructionMark = ";";

    // Visibility
    public static final String Public = "public";
    public static final String Private = "private";
    
    // Modifiers
    public static final String Final = "final";
    
    // Vars
    public static final String ExitCodeVar = "_exitCode";
    
    // Functions
    public static final String EntryPointFn = "main";
    
    // Satements
    public static final String Import = "import";
    public static final String While = "while";
    public static final String For = "for";
    public static final String If = "if";
    public static final String Else = "else";
    public static final String Return = "return";
    public static final String Do = "do";
    public static final String Break = "break";
    public static final String Continue = "continue";
    public static final String Switch = "switch";
    public static final String Case = "case";
    public static final String Default = "default";

    // types
    public static final String Void = "void";
    public static final String Int = "int";
    public static final String Double = "double";
    public static final String Char = "char";
    public static final String Bool = "boolean";
    public static final String Bool2 = "bool";
    public static final String Any = "def";
    public static final String[] types = {
        Void, Int, Double, Char, Bool, Any
    };
    
    public static final String[] NativeTypes = {
        "", IntType.Name, DoubleType.Name, CharType.Name, BoolType.Name, AnyType.Name
    };
    
    // literals
    public static final String True = "true";
    public static final String False = "false";
    
    // Operators
    public static final String AssignOperator = "=";
    public static final String AddOperator = "+";
    public static final String SubOperator = "-";
    public static final String MulOperator = "*";
    public static final String DivOperator = "/";
    public static final String ModOperator = "%";
    public static final String NotOperator = "!";
    public static final String EqOperator = "==";
    public static final String NeqOperator = "!=";
    public static final String GtOperator = ">";
    public static final String LtOperator = "<";
    public static final String GteOperator = ">=";
    public static final String LteOperator = "<=";
    public static final String AndOperator = "&&";
    public static final String OrOperator = "||";
    public static final String NewOperator = "new";
    
    // Reserved identifiers
    public static final String ReservedIdentifiers =
              ' ' + Public + ' '
            + ' ' + Private + ' '
            + ' ' + Final + ' '
            + ' ' + Import + ' '
            + ' ' + Do + ' '
            + ' ' + While + ' '
            + ' ' + For + ' '
            + ' ' + If + ' '
            + ' ' + Else + ' '
            + ' ' + Return + ' '
            + ' ' + Void + ' '
            + ' ' + Int + ' '
            + ' ' + Double + ' '
            + ' ' + Char + ' '
            + ' ' + Bool + ' '
            + ' ' + Bool2 + ' '
            + ' ' + True + ' '
            + ' ' + False + ' '
            + ' ' + Any + ' '
            + ' ' + Break + ' '
            + ' ' + Continue + ' '
            + ' ' + Switch + ' '
            + ' ' + Case + ' '
            + ' ' + Default + ' '
            + ' ' + ExitCodeVar + ' '
            + ' ' + NewOperator + ' '
    ;
    
    public static final String OperatorChars = "*+/%-<>=!&|";
    public static final String BinaryOperators = 
              ' ' + AddOperator + ' '
            + ' ' + SubOperator + ' '
            + ' ' + MulOperator + ' '
            + ' ' + DivOperator + ' '
            + ' ' + ModOperator + ' '
            + ' ' + EqOperator  + ' '
            + ' ' + NeqOperator + ' '
            + ' ' + GtOperator  + ' '
            + ' ' + LtOperator  + ' '
            + ' ' + GteOperator + ' '
            + ' ' + LteOperator  + ' '
            + ' ' + AndOperator  + ' '
            + ' ' + OrOperator  + ' '
    ;
    
    public static final String ConditionalOperators = 
              ' ' + NotOperator + ' '
            + ' ' + EqOperator  + ' '
            + ' ' + NeqOperator + ' '
            + ' ' + GtOperator  + ' '
            + ' ' + LtOperator  + ' '
            + ' ' + GteOperator + ' '
            + ' ' + LteOperator  + ' '
            + ' ' + AndOperator  + ' '
            + ' ' + OrOperator  + ' '
    ;
    
    public static final String PreferencedConditionalOperator = 
              ' ' + AndOperator  + ' '
            + ' ' + OrOperator  + ' '
    ;
    
    public static final String AsgIncByOneOperator = "++";
    public static final String AsgDecByOneOperator = "--";
    public static final String AsgIncByXOperator = "+=";
    public static final String AsgDecByXOperator = "-=";
    public static final String AsgMulByXOperator = "*=";
    public static final String AsgDivByXOperator = "/=";
    
    public static final String AdvancedOperators =
              ' ' + AsgIncByOneOperator + ' '
            + ' ' + AsgDecByOneOperator + ' '
            + ' ' + AsgIncByXOperator + ' '
            + ' ' + AsgDecByXOperator + ' '
            + ' ' + AsgMulByXOperator + ' '
            + ' ' + AsgDivByXOperator + ' '
    ;
    
    /**
     * Decides whether or not the token is a type in the programming language
     * @param token The token to be used to decide.
     * @return  True if the token contains the name of a type; false otherwise.
     */
    public static boolean isType(String token)
    {
        boolean toret = false;
        
        // Manage bool specific case
        token = token.trim();
        if ( token.equals( Bool2 ) ) {
            token = Bool;
        }
        
        for(String type: types) {
            if ( type.equals( token ) ) {
                toret = true;
                break;
            }
        }
        
        return toret;
    }
    
    public static String cnvtTypeToNative(String strType)
    {
        String toret = "";
        String brackets = "";
        
        // Prepare string type
        strType = strType.trim();
        int sqbrkPos = strType.indexOf( '[' );
        
        if ( sqbrkPos >= 0 ) {
            brackets = strType.substring( sqbrkPos );
            strType = strType.substring( 0, sqbrkPos ).trim();
        }
        
        if ( strType.equals( Bool2 ) ) {
            strType = Bool;                         // Manage bool specific case
        }
        
        // Lookup in types
        for(int i = 0; i < types.length; ++i) {
            if ( types[ i ].equals( strType ) ) {
                toret = NativeTypes[ i ];
                toret = toret + brackets;
                break;
            }
        }
        
        return toret;
    }
    
    public static String cnvtNativeToType(String strType)
    {
        String toret = "";
        String brackets = "";
        
        // Prepare string type
        strType = strType.trim();
        int sqbrkPos = strType.indexOf( '[' );
        
        if ( sqbrkPos >= 0 ) {
            brackets = strType.substring( sqbrkPos );
            strType = strType.substring( 0, sqbrkPos ).trim();
        }
        
        // Lookup in types
        for(int i = 0; i < NativeTypes.length; ++i) {
            if ( NativeTypes[ i ].equals( strType ) ) {
                toret = types[ i ];
                toret = toret + brackets;
                break;
            }
        }
        
        return toret;
    }
    
    public static String cnvtNativeToType(Type type)
    {
        String strType = "";
        
        if ( type != null ) {
            strType = type.toString();
        }
        
        return cnvtNativeToType( strType );
    }
}
