// L18n.java
/*
 * Manages localization different languages
 */

package jcv.core;

import core.I18n.Language;

/**
 * Aid class for localization of the IDE
 * @author baltasarq
 */
public class L18n {

    public enum Msg {
        Program,
        Module,

        MenuFile,
        OptNew,
        OptOpen,
        OptSave,
        OptClose,
        OptRefresh,
        OptQuit,

        MenuEdit,
        OptCopy,
        OptCut,
        OptPaste,
        OptFind,
        OptFindAgain,
        OptIndentMore,
        OptIndentLess,
        OptPreferences,
        
        MenuView,
        OptViewShortcuts,
        OptViewStatusbar,
        OptViewLineNumbers,
        OptChangeView,

        MenuZoom,
        OptIncZoom,
        OptDecZoom,

        MenuExecute,
        OptRun,
        OptDoc,

        MenuHelp,
        OptContents,
        OptAbout,

        RunOnlyPrograms,
        Executing,
        ExecutedExitCode,
        CloseWhenReady,
        MissingFile,
        IOError,
        Unknown,
        Retrieving,
        NoProject,
        Search,
        TextNotFound,
        Ready,
        Compiling,
    };

    public static final String[] StrMsgEN  = {
        "Program",
        "Module",

        "File",
        "New",
        "Open",
        "Save",
        "Close",
        "Refresh",
        "Quit",

        "Edit",
        "Copy",
        "Cut",
        "Paste",
        "Find",
        "Find Again",
        "Indent more",
        "Indent less",
        "Preferences",

        "View",
        "View toolbar",
        "View status bar",
        "View line numbers",
        "Change view...",

        "Zoom",
        "Increment zoom",
        "Decrement zoom",

        "Project",
        "Run",
        "Documentation",

        "Help",
        "Contents",
        "About",

        "Only programs can be executed.",
        "Executing",
        "Program finished with exit code ",
        "Close this window when ready",
        "Missing file",
        "Unexpected I/O exception",
        "unknown",
        "retrieving",
        "No project",
        "Search",
        "Text was not found",
        "Ready",
        "compiling",
    };

    public static final String[] StrMsgES  = {
        "Programa",
        "Módulo",

        "Archivo",
        "Nuevo",
        "Abrir",
        "Guardar",
        "Cerrar",
        "Refrescar",
        "Salir",

        "Editar",
        "Copiar",
        "Cortar",
        "Pegar",
        "Buscar",
        "Buscar de nuevo",
        "Incrementar indentado",
        "Decrementar indentado",
        "Preferencias",

        "Ver",
        "Ver barra de herramientas",
        "Ver barra de estado",
        "Ver números de línea",
        "Cambia vista...",

        "Zoom",
        "Incrementar zoom",
        "Decrementar zoom",

        "Proyecto",
        "Ejecutar",
        "Documentos",

        "Ayuda",
        "Contenido",
        "Acerca de...",

        "Imposible ejecutar si no es programa.",
        "Ejecutando",
        "Programa finalizado con valor de salida ",
        "Cierre esta ventana para continuar",
        "No se encuentra el archivo,",
        "Error de entrada/salida inesperado",
        "desconocido",
        "recuperando",
        "Sin proyecto",
        "Buscar",
        "Texto no encontrado",
        "Preparado",
        "compilando",
    };

    public static String[] StrMsg = StrMsgES;
    
    public static String[][] StrMsgs = {
        StrMsgEN, StrMsgES
    };

    public static String getMsg(Msg msg)
    {
        return getMsg( core.I18n.language, msg );
    }

    public static String getMsg(Language lang, Msg msg)
    {
        return StrMsgs[ lang.ordinal() ][ msg.ordinal() ];
    }

}
