/*
 * DlgPreferences.java
 * @author jbgarcia@uvigo.es
 * @date 16-oct-2010, 16:50:06
 */

package jcv.ui;

import core.I18n;
import core.I18n.Language;
import core.Parser;
import core.Compiler;

import javax.swing.*;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

/**
 * Opens a simple dialog for the user to choose the language to use.
 * @author baltasarq
 */
public class DlgPreferences extends javax.swing.JDialog {
    public static final String[] LabelsEN = {
            "Preferences", "Language:", "Close", "Beginner", "Expert", "Mode", "Target"
    };
    public static final String[] LabelsES = {
            "Preferencias", "Idioma:", "Cerrar", "Principiante", "Experto", "Modo", "Plataforma"
    };
    public static final String[][] Labels = { LabelsEN, LabelsES };
    public enum Etq { Title, Label, Button, Beginner, Expert, TitleMode, TitleTarget  };

    /** Creates new form DlgPreferences */
    public DlgPreferences(java.awt.Frame parent, Parser.Mode mode, Compiler.OutputPlatform target)
    {
        super( parent, true );              // Modal dialog

        this.setLanguage( core.I18n.language );
        this.mode = mode;
        this.target = target;

        this.buildGui();
        this.setIconImage( parent.getIconImage() );
        this.setLocationRelativeTo( parent );
        this.setResizable( false );

        // Fill labels
        this.translateInto( this.getLanguage() );
    }

    private void translateInto(Language language)
    {
        this.setLanguage( language );
        this.setTitle( labels[ Etq.Title.ordinal() ] );
        this.lbLanguage.setText( labels[ Etq.Label.ordinal() ] );
        this.btClose.setText( labels[ Etq.Button.ordinal() ] );
        this.rbBeginner.setText( labels[ Etq.Beginner.ordinal() ] );
        this.rbExpert.setText( labels[ Etq.Expert.ordinal() ] );
        this.tbMode.setTitle( labels[ Etq.TitleMode.ordinal() ] );
        this.tbTarget.setTitle( labels[ Etq.TitleTarget.ordinal() ] );

        if ( language == Language.EN ) {
            this.rbBeginner.setMnemonic( KeyEvent.VK_B );
        } else {
            this.rbBeginner.setMnemonic( KeyEvent.VK_P );
        }

        return;
    }

    private void buildGui()
    {
        // Prepare form
        this.setDefaultCloseOperation( javax.swing.WindowConstants.DISPOSE_ON_CLOSE );
        this.setModal( true );

        // Create widgets
        this.cbLanguage = new javax.swing.JComboBox();
        this.btClose = new javax.swing.JButton();
        this.lbLanguage = new javax.swing.JLabel();

        // Prepare panel for language
        this.lbLanguage.setText( "Lang:" );
        this.cbLanguage.setModel( new DefaultComboBoxModel( I18n.StrLanguages[ this.getLanguage().ordinal() ] ) );
        this.cbLanguage.setSelectedIndex( this.getLanguage().ordinal() );
        this.cbLanguage.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                cbLanguageItemStateChanged(evt);
            }
        });
        JPanel pnlLang = new JPanel( new BorderLayout( 10, 10 ) );
        pnlLang.add( this.lbLanguage, BorderLayout.WEST );
        pnlLang.add( this.cbLanguage, BorderLayout.CENTER );
        pnlLang.setBorder( new EmptyBorder( 5, 5, 5, 5 ) );
        this.btClose.setIcon( new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconClose.gif" ) ) );
        this.btClose.setText( "X" );
        this.btClose.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onClose(evt);
            }
        } );
        JPanel pnlButton = new JPanel( new BorderLayout( 10, 10 ) );
        pnlButton.add( this.btClose, BorderLayout.EAST );
        pnlButton.setBorder( new EmptyBorder( 5, 5, 5, 5 ) );

        // Prepare radio button panel for compilation mode
        JPanel pnlMode = new JPanel( new GridLayout( 0, 1 ) );
        this.rbBeginner = new JRadioButton( "Beginner" );
        this.rbBeginner.setMnemonic( KeyEvent.VK_B );
        this.rbBeginner.addActionListener( new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {DlgPreferences.this.mode = Parser.Mode.Beginner;
            }
        });
        this.rbExpert = new JRadioButton( "Expert" );
        this.rbExpert.setMnemonic( KeyEvent.VK_E );
        this.rbExpert.addActionListener( new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                DlgPreferences.this.mode = Parser.Mode.Expert;
            }
        });
        this.bgRadioGroupMode = new ButtonGroup();
        this.bgRadioGroupMode.add( this.rbBeginner );
        this.bgRadioGroupMode.add( this.rbExpert );
        this.bgRadioGroupMode.clearSelection();
        this.rbBeginner.setSelected( this.mode == Parser.Mode.Beginner );
        this.rbExpert.setSelected( this.mode == Parser.Mode.Expert );
        pnlMode.add( this.rbBeginner );
        pnlMode.add( this.rbExpert );
        pnlMode.setBorder( this.tbMode = new TitledBorder( "Compilation mode"  ) );

        // Prepare radio button panel for target platform
        JPanel pnlTarget = new JPanel( new GridLayout( 0, 1 ) );
        this.rbJavaScript = new JRadioButton( "JavaScript" );
        this.rbJavaScript.setMnemonic( KeyEvent.VK_J );
        this.rbJavaScript.addActionListener( new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {DlgPreferences.this.target = Compiler.OutputPlatform.Js;
            }
        });
        this.rbBsh = new JRadioButton( "Bsh" );
        this.rbBsh.setMnemonic( KeyEvent.VK_H );
        this.rbBsh.addActionListener( new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                DlgPreferences.this.target = Compiler.OutputPlatform.Bsh;
            }
        });
        this.bgRadioGroupTarget = new ButtonGroup();
        this.bgRadioGroupTarget.add( this.rbJavaScript );
        this.bgRadioGroupTarget.add( this.rbBsh );
        this.bgRadioGroupTarget.clearSelection();
        this.rbJavaScript.setSelected( this.target == Compiler.OutputPlatform.Js );
        this.rbBsh.setSelected( this.target == Compiler.OutputPlatform.Bsh  );
        pnlTarget.add( this.rbJavaScript );
        pnlTarget.add( this.rbBsh );
        pnlTarget.setBorder( tbTarget = new TitledBorder( "Target platform" ) );

        JPanel pnlModeTarget = new JPanel( new GridLayout( 0, 1 ) );
        pnlModeTarget.add( pnlMode );
        pnlModeTarget.add( pnlTarget );

        // Layout
        this.getContentPane().setLayout( new BoxLayout( this.getContentPane(), BoxLayout.PAGE_AXIS ) );
        this.getContentPane().add( pnlLang );
        this.getContentPane().add( pnlModeTarget );
        this.getContentPane().add( pnlButton );

        // Final polishing
        this.pack();
        this.setMinimumSize( new Dimension( pnlLang.getWidth() * 2, ( pnlLang.getHeight() + pnlButton.getHeight() ) ) );
    }

    private void onClose(java.awt.event.ActionEvent evt)
    {
        this.setVisible( false );
    }

    private void cbLanguageItemStateChanged(java.awt.event.ItemEvent evt)
    {
        int option = cbLanguage.getSelectedIndex();

        if ( option >= 0 ) {
            this.translateInto( Language.values()[ option ] );
        }
    }

    /**
     * @return the language chosen
     */
    public Language getLanguage()
    {
        return I18n.language;
    }

    /**
     * @param language the language to set
     */
    private void setLanguage(Language language) {
        I18n.language = language;
        this.labels = Labels[ this.getLanguage().ordinal() ];
    }

    public Parser.Mode getMode()
    {
        return this.mode;
    }

    public Compiler.OutputPlatform getTarget()
    {
        return this.target;
    }

    private JButton btClose;
    private JComboBox cbLanguage;
    private JLabel lbLanguage;
    private JRadioButton rbBeginner;
    private JRadioButton rbExpert;
    private ButtonGroup bgRadioGroupMode;
    private JRadioButton rbJavaScript;
    private JRadioButton rbBsh;
    private ButtonGroup bgRadioGroupTarget;
    private TitledBorder tbMode;
    private TitledBorder tbTarget;

    private String[] labels;
    private Parser.Mode mode;
    private Compiler.OutputPlatform target;
}
