// MainGui.java
/*
    MainGui, the main Gui of the app
*/

package jcv.ui;

import bsh.EvalError;
import bsh.Interpreter;
import bsh.util.JConsole;
import core.*;
import core.Compiler;
import core.I18n.Language;
import core.intermediatecode.CodeUnit;
import core.intermediatecode.Prg;
import core.lexical.Reserved;
import jcv.core.AppInfo;
import jcv.core.L18n;
import jcv.core.L18n.Msg;

import javax.swing.*;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.text.BadLocationException;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.io.*;
import java.net.URISyntaxException;
import java.net.URLDecoder;

/**
 * The main form
 * @author baltasarq
 */
public class MainGui extends javax.swing.JFrame {

    public static final int CodePage = 0;
    public static final int DocPage = 1;

    public static final String EtqDirRes = "jcv/res";
    public static final String DocDir = "doc";
    public static final String EtqIconName = "jc.png";
    public static final String EtqLanguage = "LANGUAGE";
    public static final String EtqWidth = "WIDTH";
    public static final String EtqHeight = "HEIGHT";
    public static final String EtqToolbar = "TOOLBAR";
    public static final String EtqMode = "MODE";
    public static final String EtqTarget = "TARGET";
    public static final String ConfigurationFile = ".jc.ini";
    public static final int BaseFontSize = 16;
    public static final Color KeywordsColor = new Color( 1, 1, 129 );
    public static final Color RemarkTextColor = new Color( 1, 113, 1 );
    public static final Color BackgroundColor = Color.white;
    public static final Color ForegroundColor = Color.black;
    public static final Color CaretColor = Color.red;
    public static final double DividerLocation = 0.8d;
    public static final String ProgramFileExt = Prg.ProgramFileExt;
    public static final String[] FileExts = {
            ProgramFileExt, Compiler.ModuleFileExt
    };
    public static final String[] FileExplanations = {
        "jc prog.", "jc mod."
    };

    public static final String[] ReservedWords = {
        "public", "private", "final",
        "int", "void", "double", "char", "boolean",
        "new", "if", "else", "switch", "case",
        "for", "break", "continue", "while", "do",
        "return", "def"
    };

    public static final String[] RemarkTokens = {
        "import", "[", "]", "{", "}", "/*", "*/", "//",
        "true", "false", "default",
        "'", "\"", "(", ")",
        "++", "--", "+=", "*=", "/=", "-=",
        "///", "/**", "**/",
        "+", "-", "*", "/",
        "<", ">", "=", "==", "<=", ">=", "!=",
        "@name", "@param", "@return", "@brief", "@date", "@author"

    };

    public static final boolean ShowConsole = true;

    /** Creates new form MainGui */
    public MainGui()
    {
        this.buildGui();
        this.setLanguage( Language.ES );
        this.translateIntoCurrentLanguage();
        this.loadConfiguration();

        // Set icon
        try {
            ImageIcon iconImg = new ImageIcon(
                           Toolkit.getDefaultToolkit().getImage(
                                MainGui.class.getResource( "/" + EtqDirRes + "/" + EtqIconName ) )
            );
            this.setIconImage( iconImg.getImage() );
        } catch(Exception exc)
        {
            showErrorMsg( "Unable to set icon: " + exc.getMessage() );
        }

        this.setPaths();
    }

    private void setPaths()
    {
        File resDir;

        try {
            // Set root dir
            String dir = getClass().getProtectionDomain().getCodeSource().getLocation().getPath();
            dir = URLDecoder.decode( dir, "UTF-8" );
            resDir = new File( dir ).getCanonicalFile();

            if ( !resDir.isDirectory() ) {
                resDir = resDir.getParentFile();
            }
        } catch(IOException exc)
        {
            resDir = new File( "." );
        }

        // Set lib dir
        this.libDir = new File( resDir, LibLoader.LibraryPathPrefix );

        if ( !this.libDir.exists() ) {
            this.libDir = new File( ".", LibLoader.LibraryPathPrefix );
        }

        // Set help dir
        this.docDir = new File( resDir, DocDir );

        if ( !this.docDir.exists() ) {
            this.docDir = new File( ".", DocDir );
        }

        return;
    }

    public String getSearchText()
    {
        return this.searchText;
    }

    @Override
    public void setSize(Dimension d)
    {
        if ( txtSource != null ) {
            Dimension txtSize = txtSource.getSize();

            if ( d.width < txtSize.width ) {
                // don't let the Textpane get sized differently than its parent
                super.setSize( txtSize.width, d.height );
            } else {
                super.setSize( d );
            }
        }

        return;
    }

    private void buildToolbar()
    {
        this.tbTools = new javax.swing.JToolBar();
        this.btNew = new javax.swing.JButton();
        this.btOpen = new javax.swing.JButton();
        this.btSave = new javax.swing.JButton();
        this.btCopy = new javax.swing.JButton();
        this.btCut = new javax.swing.JButton();
        this.btPaste = new javax.swing.JButton();
        this.btRun = new javax.swing.JButton();

        this.tbTools.setBorder( javax.swing.BorderFactory.createBevelBorder( javax.swing.border.BevelBorder.RAISED ) );
        this.tbTools.setFloatable( false );
        this.tbTools.setRollover( true );

        this.btNew.setIcon( new javax.swing.ImageIcon(getClass().getResource( "/jcv/res/iconNew24.gif" ) ) );
        this.btNew.setFocusable( false );
        this.btNew.setHorizontalTextPosition( javax.swing.SwingConstants.CENTER );
        this.btNew.setVerticalTextPosition( javax.swing.SwingConstants.BOTTOM );
        this.btNew.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onNew(evt);
            }
        } );
        this.tbTools.add( this.btNew );

        this.btOpen.setIcon( new javax.swing.ImageIcon( getClass().getResource( "/jcv/res/iconOpen24.gif" ) ) );
        this.btOpen.setFocusable( false );
        this.btOpen.setHorizontalTextPosition( javax.swing.SwingConstants.CENTER );
        this.btOpen.setVerticalTextPosition( javax.swing.SwingConstants.BOTTOM );
        this.btOpen.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onOpen(evt);
            }
        } );
        this.tbTools.add( this.btOpen );

        this.btSave.setIcon( new javax.swing.ImageIcon( getClass().getResource("/jcv/res/iconSave24.gif")));
        this.btSave.setFocusable( false );
        this.btSave.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        this.btSave.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        this.btSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onSave(evt);
            }
        });
        this.tbTools.add( this.btSave );
        this.tbTools.add( new javax.swing.JToolBar.Separator() );

        this.btCopy.setIcon( new javax.swing.ImageIcon( getClass().getResource("/jcv/res/iconCopy24.gif") ) );
        this.btCopy.setFocusable( false );
        this.btCopy.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        this.btCopy.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        this.btCopy.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onCopy(evt);
            }
        });
        this.tbTools.add( this.btCopy );

        this.btCut.setIcon( new javax.swing.ImageIcon( getClass().getResource("/jcv/res/iconCut24.gif") ) );
        this.btCut.setFocusable( false );
        this.btCut.setHorizontalTextPosition( javax.swing.SwingConstants.CENTER );
        this.btCut.setVerticalTextPosition( javax.swing.SwingConstants.BOTTOM );
        this.btCut.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onCut(evt);
            }
        } );
        this.tbTools.add( this.btCut );

        this.btPaste.setIcon( new javax.swing.ImageIcon( getClass().getResource("/jcv/res/iconPaste24.gif") ) );
        this.btPaste.setFocusable( false );
        this.btPaste.setHorizontalTextPosition( javax.swing.SwingConstants.CENTER );
        this.btPaste.setVerticalTextPosition( javax.swing.SwingConstants.BOTTOM );
        this.btPaste.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onPaste(evt);
            }
        } );
        this.tbTools.add( this.btPaste );
        this.tbTools.add( new javax.swing.JToolBar.Separator() );

        this.btRun.setIcon(new javax.swing.ImageIcon( getClass().getResource("/jcv/res/iconPlay24.gif") ) );
        this.btRun.setFocusable(false);
        this.btRun.setHorizontalTextPosition( javax.swing.SwingConstants.CENTER );
        this.btRun.setVerticalTextPosition( javax.swing.SwingConstants.BOTTOM );
        this.btRun.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onRun(evt);
            }
        } );
        this.tbTools.add( this.btRun );
        this.tbTools.add( new javax.swing.JToolBar.Separator() );
    }

    private void buildStatusBar()
    {
        this.tbStatus = new JToolBar();
        this.lblPosition = new JLabel();
        this.lblStatus = new JLabel();
        this.tbStatus.setBorder( javax.swing.BorderFactory.createBevelBorder( javax.swing.border.BevelBorder.LOWERED ) );
        this.tbStatus.setFloatable( false );
        this.tbStatus.setRollover( true );

        this.lblPosition.setText( "0:0" );
        this.tbStatus.add( this.lblPosition );
        this.tbStatus.add( new JLabel( " | ") );
        this.tbStatus.add( this.lblStatus );
    }

    private void buildMenu()
    {
        mMainMenu = new javax.swing.JMenuBar();
        mFile = new javax.swing.JMenu();
        mNew = new javax.swing.JMenu();
        opNewProgram = new javax.swing.JMenuItem();
        opNewModule = new javax.swing.JMenuItem();
        opOpen = new javax.swing.JMenuItem();
        opSave = new javax.swing.JMenuItem();
        opClose = new javax.swing.JMenuItem();
        opQuit = new javax.swing.JMenuItem();
        mEdit = new javax.swing.JMenu();
        opCopy = new javax.swing.JMenuItem();
        opCut = new javax.swing.JMenuItem();
        opPaste = new javax.swing.JMenuItem();
        opFind = new javax.swing.JMenuItem();
        opFindAgain = new javax.swing.JMenuItem();
        opIndentMore = new javax.swing.JMenuItem();
        opIndentLess = new javax.swing.JMenuItem();
        opPreferences = new javax.swing.JMenuItem();
        mView = new javax.swing.JMenu();
        opViewShortcuts = new javax.swing.JCheckBoxMenuItem();
        opViewStatusBar = new javax.swing.JCheckBoxMenuItem();
        opViewLineNumbers = new javax.swing.JCheckBoxMenuItem();
        opChangeView = new javax.swing.JMenuItem();
        mZoom = new javax.swing.JMenu();
        opIncZoom = new javax.swing.JMenuItem();
        opDecZoom = new javax.swing.JMenuItem();
        mExecute = new javax.swing.JMenu();
        opRun = new javax.swing.JMenuItem();
        opDocument = new javax.swing.JMenuItem();
        mHelp = new javax.swing.JMenu();
        opHelp = new javax.swing.JMenuItem();
        opAbout = new javax.swing.JMenuItem();

        mFile.setMnemonic('f');
        mFile.setText("File");
        mFile.setName("mFile"); 

        mNew.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconNew.gif"))); 
        mNew.setText("New");

        opNewProgram.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, java.awt.event.InputEvent.CTRL_MASK));
        opNewProgram.setMnemonic('p');
        opNewProgram.setText("Program");
        opNewProgram.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onNew(evt);
            }
        });
        mNew.add(opNewProgram);

        opNewModule.setMnemonic('m');
        opNewModule.setText("Module");
        opNewModule.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onNewModule(evt);
            }
        });
        mNew.add(opNewModule);

        mFile.add(mNew);

        opOpen.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        opOpen.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconOpen.gif"))); 
        opOpen.setMnemonic('o');
        opOpen.setText("Open");
        opOpen.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onOpen(evt);
            }
        });
        mFile.add(opOpen);

        opSave.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        opSave.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconSave.gif"))); 
        opSave.setMnemonic('S');
        opSave.setText("Save");
        opSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onSave(evt);
            }
        });
        mFile.add(opSave);

        opClose.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconClose.gif"))); 
        opClose.setMnemonic('c');
        opClose.setText("Close");
        opClose.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onClose(evt);
            }
        });
        mFile.add(opClose);

        opQuit.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Q, java.awt.event.InputEvent.CTRL_MASK));
        opQuit.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconExit.gif"))); 
        opQuit.setMnemonic('q');
        opQuit.setText("Quit");
        opQuit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                quit();
            }
        });
        mFile.add(opQuit);

        mMainMenu.add(mFile);

        mEdit.setMnemonic('e');
        mEdit.setText("Edit");
        mEdit.setName("Edit");

        opCopy.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, java.awt.event.InputEvent.CTRL_MASK));
        opCopy.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconCopy.gif"))); 
        opCopy.setMnemonic('c');
        opCopy.setText("Copy");
        opCopy.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onCopy(evt);
            }
        });
        mEdit.add(opCopy);

        opCut.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_X, java.awt.event.InputEvent.CTRL_MASK));
        opCut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconCut.gif"))); 
        opCut.setMnemonic('t');
        opCut.setText("Cut");
        opCut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onCut(evt);
            }
        });
        mEdit.add(opCut);

        opPaste.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_V, java.awt.event.InputEvent.CTRL_MASK));
        opPaste.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconPaste.gif"))); 
        opPaste.setMnemonic('p');
        opPaste.setText("Paste");
        opPaste.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onPaste(evt);
            }
        });
        mEdit.add(opPaste);
        mEdit.add( new JSeparator() );

        opFind.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F, java.awt.event.InputEvent.CTRL_MASK));
        opFind.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconFind.gif"))); 
        opFind.setMnemonic('f');
        opFind.setText("Find...");
        opFind.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onFind(evt);
            }
        });
        mEdit.add(opFind);

        opFindAgain.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F3, 0));
        opFindAgain.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconFindAgain.gif"))); 
        opFindAgain.setMnemonic('a');
        opFindAgain.setText("Find again...");
        opFindAgain.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onFindAgain(evt);
            }
        });
        mEdit.add(opFindAgain);
        mEdit.add( new JSeparator() );

        opIndentMore.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_TAB, 0));
        opIndentMore.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconIndentMore.gif"))); 
        opIndentMore.setMnemonic('m');
        opIndentMore.setText("Indent more");
        opIndentMore.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onIndentMore(evt);
            }
        });
        mEdit.add(opIndentMore);

        opIndentLess.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_TAB, java.awt.event.InputEvent.SHIFT_MASK));
        opIndentLess.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconIndentLess.gif"))); 
        opIndentLess.setMnemonic('l');
        opIndentLess.setText("Indent less");
        opIndentLess.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onIndentLess(evt);
            }
        });
        mEdit.add(opIndentLess);
        mEdit.add( new JSeparator() );

        opPreferences.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F2, 0));
        opPreferences.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconPreferences.gif"))); 
        opPreferences.setMnemonic('e');
        opPreferences.setText("Preferences");
        opPreferences.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onShowPreferences(evt);
            }
        });
        mEdit.add(opPreferences);

        mMainMenu.add( mEdit );

        mView.setMnemonic( 'v' );
        mView.setText( "View" );

        opViewShortcuts.setMnemonic('s');
        opViewShortcuts.setSelected(true);
        opViewShortcuts.setText("View shortcuts");
        opViewShortcuts.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onViewToolbar(evt);
            }
        });
        mView.add(opViewShortcuts);

        opViewStatusBar.setSelected(true);
        opViewStatusBar.setText("View Status bar");
        opViewStatusBar.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onViewStatusbar(evt);
            }
        });
        mView.add(opViewStatusBar);

        opViewLineNumbers.setMnemonic('n');
        opViewLineNumbers.setSelected(true);
        opViewLineNumbers.setText("View line numbers");
        opViewLineNumbers.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onViewLineNumbers(evt);
            }
        });
        mView.add(opViewLineNumbers);

        opChangeView.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F7, 0));
        opChangeView.setText("Change view...");
        opChangeView.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onChangeView(evt);
            }
        });
        mView.add(opChangeView);

        mZoom.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconZoom.gif"))); 
        mZoom.setMnemonic('z');
        mZoom.setText("Zoom");

        opIncZoom.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_PLUS, java.awt.event.InputEvent.CTRL_MASK));
        opIncZoom.setMnemonic('i');
        opIncZoom.setText("Increment font");
        opIncZoom.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onIncZoom(evt);
            }
        });
        mZoom.add(opIncZoom);

        opDecZoom.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_MINUS, java.awt.event.InputEvent.CTRL_MASK));
        opDecZoom.setMnemonic('d');
        opDecZoom.setText("Decrement font");
        opDecZoom.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onDecZoom(evt);
            }
        });
        mZoom.add(opDecZoom);

        mView.add(mZoom);

        mMainMenu.add(mView);

        mExecute.setMnemonic('p');
        mExecute.setText("Project");

        opRun.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F5, 0));
        opRun.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconPlay.gif"))); 
        opRun.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onRun(evt);
            }
        });
        mExecute.add(opRun);

        opDocument.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F4, 0));
        opDocument.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconDocumentation.png"))); 
        opDocument.setMnemonic('d');
        opDocument.setText("Documentation");
        opDocument.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onShowDocumentation( evt );
            }
        });
        mExecute.add(opDocument);

        mMainMenu.add(mExecute);

        mHelp.setMnemonic('h');
        mHelp.setText("Help");

        opHelp.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F1, 0));
        opHelp.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconHelp.gif"))); 
        opHelp.setMnemonic('h');
        opHelp.setText("Help");
        opHelp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onHelp(evt);
            }
        });
        mHelp.add(opHelp);

        opAbout.setIcon(new javax.swing.ImageIcon(getClass().getResource("/jcv/res/iconAbout.gif"))); 
        opAbout.setMnemonic('a');
        opAbout.setText("About");
        opAbout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onAbout(evt);
            }
        });
        mHelp.add(opAbout);

        mMainMenu.add(mHelp);

        this.setJMenuBar( mMainMenu );
    }

    private void buildSearchBar()
    {
        this.tbSearch = new javax.swing.JToolBar();
        this.edSearchText = new javax.swing.JTextField();
        this.btFind = new javax.swing.JButton();

        this.tbSearch.setBorder( javax.swing.BorderFactory.createBevelBorder( javax.swing.border.BevelBorder.RAISED ) );
        this.tbSearch.setFloatable( false );
        this.tbSearch.setRollover( true );

        this.edSearchText.setForeground( new java.awt.Color( 153, 153, 153 ) );
        this.edSearchText.setText( "Search..." );
        this.edSearchText.addFocusListener (new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                onSearchFieldFocusLost(evt);
            }
        });

        this.edSearchText.addKeyListener( new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                onSearchFieldKeyReleased(evt);
            }
        });

        this.tbSearch.add( this.edSearchText );
        this.tbSearch.add( new javax.swing.JToolBar.Separator() );

        this.btFind.setIcon( new javax.swing.ImageIcon( getClass().getResource( "/jcv/res/iconFind24.gif") ) );
        this.btFind.setToolTipText( "Look for text in the program/module" );
        this.btFind.setFocusable( false );
        this.btFind.setHorizontalTextPosition( javax.swing.SwingConstants.CENTER );
        this.btFind.setVerticalTextPosition( javax.swing.SwingConstants.BOTTOM );
        this.btFind.addActionListener( new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                onFindButtonClicked(evt);
            }
        });
        tbSearch.add( btFind );
    }

    private void buildOutput()
    {
        this.txtOutput = new javax.swing.JTextArea();
        this.txtOutput.setColumns( 20 );
        this.txtOutput.setEditable( false );
        this.txtOutput.setFont( new Font("Courier New", 2, 14 ) );
        this.txtOutput.setForeground( new Color(153, 51, 0) );
        this.txtOutput.setRows( 5 );
        this.txtOutput.setBackground( Color.lightGray );

        this.txtOutput.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                MainGui.this.onOutputMouseClicked(evt);
            }
        });

        this.spOutputScroller = new JScrollPane();
        this.spOutputScroller.setViewportView( this.txtOutput );
        this.spOutputScroller.setMinimumSize( new Dimension( 500, 150 ) );
    }

    private void buildEditor()
    {
        this.txtSource = new SourceEditor(
                ForegroundColor,
                BackgroundColor,
                CaretColor,
                KeywordsColor,
                RemarkTextColor,
                BaseFontSize,
                ReservedWords, RemarkTokens
        );

        this.txtSource.addCaretListener( new CaretListener() {
                    public void caretUpdate(CaretEvent evt) {
                        showCaretPosition();
                    }
                } );
        this.textLineNumber = new TextLineNumber( this.txtSource, 6 );

        this.spEditorScroller = new JScrollPane();
        this.spEditorScroller.setBackground( Color.lightGray );
        this.spEditorScroller.setViewportView( this.txtSource );
        this.spEditorScroller.setRowHeaderView( this.textLineNumber );
    }

    private void buildEditorDocTabs()
    {
        this.tpMainTab = new javax.swing.JTabbedPane();
        this.tpBrowse = new javax.swing.JTabbedPane();
        this.tpMainTab.setTabPlacement( javax.swing.JTabbedPane.BOTTOM );
        this.tpMainTab.addTab( "Doc", this.tpBrowse );
        this.tpMainTab.insertTab( "Cod", null, this.spEditorScroller, "Cod", 0 );
        this.tpMainTab.setMinimumSize( new Dimension( 500, 400  ) );

        try {
            this.docBrowser = DocBrowser.createDocBrowser( this.tpBrowse );
        }
        catch(IOException exc)
        {
            this.showErrorMsg( exc.getLocalizedMessage() );
        }
    }


    /** This method is called from within the constructor to
     * initialize the form.
     */
    private void buildGui()
    {
        // Prepare main window events
        setDefaultCloseOperation( javax.swing.WindowConstants.EXIT_ON_CLOSE );

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                MainGui.this.quit();
            }
        });

        // Create main widgets
        this.buildToolbar();
        this.buildStatusBar();
        this.buildMenu();
        this.buildSearchBar();
        this.buildOutput();
        this.buildEditor();
        this.buildEditorDocTabs();

        this.spPanelDivision = new JSplitPane( JSplitPane.VERTICAL_SPLIT, this.tpMainTab, this.spOutputScroller );
        this.spPanelDivision.setOneTouchExpandable( false );
        this.spPanelDivision.setResizeWeight( DividerLocation );

        // Layout
        JPanel pnlToolbars = new JPanel( new BorderLayout() );
        pnlToolbars.add( this.tbTools, BorderLayout.NORTH );
        pnlToolbars.add( this.tbSearch, BorderLayout.SOUTH );

        this.getContentPane().setLayout( new BorderLayout() );
        this.getContentPane().add( pnlToolbars, BorderLayout.NORTH );
        this.getContentPane().add( this.spPanelDivision, BorderLayout.CENTER  );
        this.getContentPane().add( this.tbStatus, BorderLayout.SOUTH );

        this.setMinimumSize( new Dimension( 620, 450 ) );
        this.pack();
        this.deactivateUI();
    }

    private void onAbout(ActionEvent evt)
    {
        this.setStatus();

        JOptionPane.showMessageDialog(
            this,
            AppInfo.getCompleteName()
                + '\n' + AppInfo.Info
                + '\n' + '\n' + AppInfo.Author,
            AppInfo.Name + "...",
            JOptionPane.CLOSED_OPTION,
            new ImageIcon( this.getIconImage() )
        );
    }

    private void onSave(ActionEvent evt)
    {
        if ( project != null ) {
            this.setStatus();
            save( project );
        }
    }

    private boolean compile(File project)
    {
        boolean toret;

        if ( this.target == Compiler.OutputPlatform.Js ) {
            toret = compileJs( project );
        } else {
            toret = compileBsh( project );
        }

        return toret;
    }

    private boolean compileBsh(File project)
    {
        boolean toret = false;
        Compiler compiler = null;

        this.setStatus();

        try {
            compiler = new BshCompiler( project, this.mode, I18n.language );
            compiler.setLibPath( this.libDir );

            if ( project.toString().endsWith( ProgramFileExt ) ) {
                compiler.compile();
                toret = true;
            } else throw new RuntimeException( L18n.getMsg( Msg.RunOnlyPrograms ) );
        }
        catch (IOException | CompilingError exc)
        {
            this.prepareOutputForErrors();
            this.txtOutput.append( AppInfo.Error + ": " + exc.getMessage() );
        }

        if ( !toret ) {
            this.setErrorStatus( L18n.getMsg( Msg.Compiling ) );
        }

        return toret;
    }

    private boolean compileJs(File project)
    {
        boolean toret = false;
        Compiler compiler = null;

        this.setStatus();

        try {
            compiler = new JsCompiler( project, this.mode, I18n.language );
            compiler.setLibPath( this.libDir );

            if ( project.toString().endsWith( ProgramFileExt ) ) {
                compiler.compile();
                toret = true;
            } else throw new RuntimeException( L18n.getMsg( Msg.RunOnlyPrograms ) );
        }
        catch (IOException | CompilingError exc)
        {
            this.prepareOutputForErrors();
            this.txtOutput.append( AppInfo.Error + ": " + exc.getMessage() );
        }

        if ( !toret ) {
            this.setErrorStatus( L18n.getMsg( Msg.Compiling ) );
        }

        return toret;
    }

    private boolean execute(File project, boolean showConsole)
    {
        boolean toret;

        if ( this.target == Compiler.OutputPlatform.Js ) {
            toret = executeHtml( project, showConsole );
        } else {
            toret = executeBsh( project, showConsole );
        }

        return toret;
    }

    private boolean executeHtml(File project, boolean showConsole)
    {
        boolean toret = false;

        this.setStatus( "Launching jC program..." );

        try {
            prepareOutputForErrors();
            String programName = project.getName();

            // Get the target file name
            String fileName = project.getAbsolutePath();
            int dotPos = fileName.lastIndexOf( '.' );
            if ( dotPos > 0 ) {
                fileName = fileName.substring( 0, dotPos );
            }
            fileName += '.' + JsCompiler.ExecutableExt;

            if ( !new File( fileName ).exists() ) {
                throw new RuntimeException( "Program has not been compiled yet." );
            } else {
                toret = true;
                Desktop.getDesktop().browse( new File( fileName ).toURI() );
            }
        } catch (FileNotFoundException ex) {
            this.txtOutput.append( L18n.getMsg( Msg.MissingFile ) + ex.getMessage() );
        } catch (IOException ex) {
            this.txtOutput.append( L18n.getMsg( Msg.IOError ) );
        } catch(RuntimeException exc) {
            this.prepareOutputForErrors();
            this.txtOutput.append(exc.getMessage());
        }

        this.setStatus();
        return toret;
    }

    private boolean executeBsh(File project, boolean showConsole)
    {
        boolean toret = false;
        Interpreter terp = null;
        JFrame consoleWindow = null;
        JTextField txtStatus = null;
        JConsole console = null;

        this.setStatus();

        try {
            prepareOutputForErrors();
            String programName = project.getName();

            // Get the interpreter file name
            String fileName = project.getAbsolutePath();
            int dotPos = fileName.lastIndexOf( '.' );
            if ( dotPos > 0 ) {
                fileName = fileName.substring( 0, dotPos );
            }
            fileName += '.' + BshCompiler.ExecutableExt;

            if ( !new File( fileName ).exists() ) {
                throw new RuntimeException( "Program has not been compiled yet." );
            }

            // Create the console window
            if ( showConsole ) {
                consoleWindow = new JFrame();
                consoleWindow.setMinimumSize( new Dimension( 600, 400) );
                consoleWindow.setSize(consoleWindow.getMinimumSize());
                consoleWindow.setLocationRelativeTo( this );
                consoleWindow.setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
                consoleWindow.setTitle( programName );
                
                // Create the console itself
                console = new JConsole();
                txtStatus = new JTextField( L18n.getMsg( Msg.Executing ) + "..." );
                JToolBar status = new JToolBar( "Status", JToolBar.HORIZONTAL );
                consoleWindow.setIconImage( this.getIconImage() );
                consoleWindow.getContentPane().add( console, BorderLayout.CENTER );
                consoleWindow.getContentPane().add( status, BorderLayout.SOUTH );
                consoleWindow.setVisible( true );
                console.setVisible( true );
                txtStatus.setVisible( true );
                txtStatus.setEditable( false );
                status.add( txtStatus );
                status.setFloatable( false );
                status.setVisible( true );
                System.setOut( console.getOut() );
                
                terp = new Interpreter( console );
           } else terp = new Interpreter();
           
            // Config. the interpreter and execute the code
            terp.setExitOnEOF( true );
            terp.source( fileName );
            toret = true;

            if ( showConsole ) {
                int errorCode =
                        ( (Integer) terp.get( Reserved.ExitCodeVar ) ).intValue()
                ;

                String msg = L18n.getMsg( Msg.ExecutedExitCode )
                        + Integer.toString( errorCode )
                        + " - "
                        + L18n.getMsg( Msg.CloseWhenReady )
                ;

                txtStatus.setText( msg );
            }
        } catch (FileNotFoundException ex) {
            this.txtOutput.append( L18n.getMsg( Msg.MissingFile ) + ex.getMessage() );
        } catch (IOException ex) {
            this.txtOutput.append( L18n.getMsg( Msg.IOError ) );
        } catch (EvalError ex) {
            String error = L18n.getMsg( Msg.Unknown );

            // Close console
            if ( consoleWindow != null ) {
                consoleWindow.setVisible( false );
                consoleWindow.dispose();
            }

            // Report error
            if ( ex != null
              && ex.getMessage() != null )
            {
                error += "\n\t" + ex.getMessage() + "\n\t" + ex.getErrorText();
            }

            txtOutput.append( AppInfo.Error + ": " + project.toString() + ": " + error );

        } catch(RuntimeException exc) {
            this.prepareOutputForErrors();
            this.txtOutput.append(exc.getMessage());
        }

        return toret;
    }

    private void onRun(ActionEvent evt)
    {
        this.setStatus();

        // Is there any project ?
        if ( project != null ) {
            this.btRun.setEnabled( false );

            try {
                onSave( null );
                prepareOutputForInfo();
                if ( compile( project ) ) {
                    execute( project, ShowConsole );
                }

                // Update documentation
                new Thread( new Runnable() {
                    @Override
                    public void run() {
                        try {
                            docBrowser.refreshAll();
                        } catch(IOException exc)
                        {
                            MainGui.this.showErrorMsg( exc.getLocalizedMessage() );
                        }
                    }
                }).start();
            } catch (Throwable exc) {
                this.prepareOutputForErrors();
                this.txtOutput.append( "Error: " + exc.toString() + '\n' );
            }

            this.btRun.setEnabled( true );
        }
        else this.setStatus( L18n.getMsg( Msg.NoProject ) );

        this.txtSource.requestFocusInWindow();
    }

    private void onOpen(ActionEvent evt)
    {
        onSave( null );

        project = SwingUtil.showOpenFileDlg( this, AppInfo.Name, FileExts, FileExplanations, project );

        if ( project != null ) {
            this.activateUI();
            this.load( project );
            this.setTitle();
        }
    }

    /**
     * Try to locate the line that caused the compiling error.
     * The user clicks on the output pane, the line number must be located inside the error message.
     */
    private void onOutputMouseClicked(MouseEvent evt)
    {
        this.setStatus();

        try {
            // Get line
            int caretPos = this.txtOutput.getCaretPosition();
            int lineNumber = this.txtOutput.getLineOfOffset( caretPos );
            int lineStart = this.txtOutput.getLineStartOffset( lineNumber );
            int lineEnd = this.txtOutput.getLineEndOffset( lineNumber );
            String line = this.txtOutput.getText().substring( lineStart, lineEnd );

            // Parse it
            line.trim();

            if ( line.indexOf( AppInfo.Error ) >= 0 ) {

                // Locate the second ':'
                int linePosBeg = line.indexOf( ':' );
                linePosBeg = line.indexOf( ':', linePosBeg + 1 );

                if ( linePosBeg >= 0 ) {
                    linePosBeg += 1;
                    int linePosEnd = linePosBeg;

                    // Look for number
                    while( Character.isDigit( line.charAt( linePosEnd ) ) ) {
                        ++linePosEnd;
                    }

                    // Get number
                    lineNumber = Integer.parseInt( line.substring( linePosBeg, linePosEnd ) );

                    caretPos = this.txtSource.getLineStartOffset( --lineNumber );
                    this.txtSource.setCaretPosition( caretPos );

                    showCaretPosition();
                    this.spOutputScroller.invalidate();
                    this.txtSource.requestFocusInWindow();
                }
                else setErrorStatus();
            }
        } catch (BadLocationException | NumberFormatException exc) {
            setErrorStatus();
        }
    }

    private void onNew(ActionEvent evt)
    {
        this.setStatus();

        onSave( null );

        File project2 = SwingUtil.showSaveFileDlg( this, AppInfo.Name, ProgramFileExt, FileExplanations[ 0 ], project );

        if ( project2 != null ) {
            if ( !project2.getAbsolutePath().endsWith( ProgramFileExt ) ) {
                this.project = new File( project2.getAbsolutePath() + "." + ProgramFileExt );
            } else {
                this.project = project2;
            }

            this.txtSource.setText( AppInfo.getProgramTemplate() );
            this.activateUI();
        }
    }

    private void onNewModule(ActionEvent evt)
    {
        File project2 = null;

        this.setStatus();
        onSave( null );

        project2 = SwingUtil.showSaveFileDlg( this, AppInfo.Name, Compiler.ModuleFileExt, FileExplanations[ 1 ], project );

        if ( project2 != null ) {
            if ( !project2.getAbsolutePath().endsWith( Compiler.ModuleFileExt ) ) {
                this.project = new File( project2.getAbsolutePath() + "." + Compiler.ModuleFileExt );
            } else {
                this.project = project2;
            }

            this.txtSource.setText( AppInfo.getModuleTemplate() );
            this.activateUI();
        }
    }

    private void onClose(ActionEvent evt)
    {
        try {
            this.onSave( null );

            this.txtSource.setFile( (File) null );
        } catch(Exception exc) {
            this.showErrorMsg( exc.getMessage() );
        }
        finally {
            this.project = null;
            this.deactivateUI();
        }
    }

    private void onHelp(ActionEvent evt)
    {
        this.setStatus();

        try {
            this.docBrowser.browse( new File( this.docDir, "index.html" ) );
            this.viewPage( DocPage );
        } catch(IOException | URISyntaxException exc)
        {
            this.showErrorMsg( exc.getLocalizedMessage() );
            this.viewPage( CodePage );
        }

        return;
    }

    private void onFind(ActionEvent evt)
    {
        this.setStatus();
        this.tbSearch.setVisible( true );
        this.edSearchText.requestFocusInWindow();
    }

    private void onFindAgain(ActionEvent evt)
    {
        this.lookFor( -1, this.getSearchText() );
    }

    private void onCopy(ActionEvent evt)
    {
        this.setStatus();
        this.txtSource.copy();
    }

    private void onCut(ActionEvent evt)
    {
        this.setStatus();
        this.txtSource.cut();
    }

    private void onPaste(ActionEvent evt)
    {
        this.setStatus();
        this.txtSource.paste();
    }

    private void onShowPreferences(ActionEvent evt)
    {
        DlgPreferences dlgPref;

        this.setStatus();

        dlgPref = new DlgPreferences( this, this.mode, this.target );
        dlgPref.setVisible( true );

        this.translateInto( dlgPref.getLanguage() );
        this.mode = dlgPref.getMode();
        this.target = dlgPref.getTarget();
    }

    private void onSearchFieldFocusLost(FocusEvent evt)
    {
        this.edSearchText.setForeground( Color.GRAY );
        this.edSearchText.setText( L18n.getMsg( Msg.Search ) + "..." );
        this.tbSearch.setVisible( false );
        this.edSearchText.selectAll();
    }

    private void onSearchFieldKeyReleased(KeyEvent evt)
    {
        if ( edSearchText.getText().isEmpty() ) {
            this.onSearchFieldFocusLost( null );
        }
        else {
            this.edSearchText.setForeground( Color.BLACK );
        }

        if ( evt.getKeyCode() == java.awt.event.KeyEvent.VK_ENTER ) {
            this.onFindButtonClicked( null );
        }
        else
        if ( evt.getKeyCode() == java.awt.event.KeyEvent.VK_ESCAPE ) {
            this.tbSearch.setVisible( false );
            this.txtOutput.requestFocusInWindow();
        }

        return;
    }

    private void onFindButtonClicked(java.awt.event.ActionEvent evt)
    {
        this.setStatus();
        this.lookFor( 0, edSearchText.getText() );
    }

    private void onDecZoom(ActionEvent evt)
    {
        this.setStatus();
        this.txtSource.setFontSize( this.txtSource.getFont().getSize() - 1 );
    }

    private void onIncZoom(ActionEvent evt)
    {
        this.setStatus();
        this.txtSource.setFontSize( this.txtSource.getFont().getSize() + 1 );
    }

    private void onViewToolbar(ActionEvent evt)
    {
        this.setStatus();
        this.tbTools.setVisible( opViewShortcuts.isSelected() );
    }

    private void onViewStatusbar(ActionEvent evt)
    {
        this.setStatus();
        this.tbStatus.setVisible( opViewStatusBar.isSelected() );
    }

    private void onIndentMore(ActionEvent evt)
    {
        this.setStatus();
        final int[] data = txtSource.getMarkedLines();
        this.txtSource.indentMore( data[ 2 ], data[ 3 ] );
    }

    private void onIndentLess(ActionEvent evt)
    {
        this.setStatus();
        final int[] data = txtSource.getMarkedLines();
        this.txtSource.indentLess( data[ 2 ], data[ 3 ] );
    }

    private void onShowDocumentation(ActionEvent evt)
    {
        this.setStatus();

        try {
            this.docBrowser.browse( new File( CodeUnit.buildDocFileName( project, true ) ) );
            this.viewPage( DocPage );
        } catch(IOException | URISyntaxException exc)
        {
            this.showErrorMsg( exc.getLocalizedMessage() );
            this.setErrorStatus();
        }

        return;
    }

    private void onViewLineNumbers(ActionEvent evt)
    {

        this.setStatus();

        if ( this.opViewLineNumbers.isSelected() ) {
            this.textLineNumber.setVisible( true );
            this.spEditorScroller.setRowHeaderView(this.textLineNumber);
        } else {
            this.textLineNumber.setVisible( false );
            this.spEditorScroller.setRowHeaderView(null);
        }

        return;
    }

    private void onChangeView(ActionEvent evt)
    {
        this.setStatus();

        if ( this.tpMainTab.getSelectedIndex() == CodePage ) {
            this.viewPage( DocPage );
        } else {
            this.viewPage( CodePage );
        }

        return;
    }

    private void activateUI()
    {
        this.prepareIDE( true );
    }

    private void deactivateUI()
    {
        this.prepareIDE( false );
    }

    private void setStatus()
    {
        this.setStatus( L18n.getMsg( Msg.Ready ) );
    }

    private void setErrorStatus()
    {
        this.setStatus( AppInfo.Error );
    }

    private void setErrorStatus(String msg)
    {
        this.setStatus( AppInfo.Error + ": " + msg );
    }

    private void setStatus(String msg)
    {
        this.lblStatus.setText( msg );
    }
    
    private void prepareIDE(boolean active)
    {
        // Main arrangements
        this.setTitle();
        this.tbStatus.setVisible( active );
        this.viewPage( CodePage );
        this.setStatus();

        // Other components
        this.spEditorScroller.setVisible( active );
        this.spOutputScroller.setVisible( active );
        this.txtOutput.setText( "" );
        this.txtOutput.setVisible(active);
        this.txtSource.setEnabled( active );
        this.txtSource.setVisible( active );
        this.edSearchText.setEnabled( active );
        this.onSearchFieldFocusLost(null);
        this.opViewStatusBar.setEnabled( active );
        this.opViewStatusBar.setSelected( active );
        this.opViewLineNumbers.setSelected( active );
        this.opViewLineNumbers.setEnabled( active );
        this.tpMainTab.setVisible( active );

        // Menus visible/not visible
        this.opSave.setEnabled( active );
        this.opClose.setEnabled( active );
        this.mExecute.setEnabled( active );
        this.mZoom.setEnabled( active );
        this.mFile.setEnabled( true );
        this.opChangeView.setEnabled( active );

        // Put only preferences active in mEdit
        for(int i = 0; i < mEdit.getItemCount(); ++i) {
            JMenuItem item = mEdit.getItem( i );

            if ( item != null ) {
                item.setEnabled( active );
            }
        }
        this.opPreferences.setEnabled( true );

        // Toolbar elements enabled / not enabled
        btSave.setEnabled( active );
        btRun.setEnabled( active );
        btFind.setEnabled( active );
        btCopy.setEnabled( active );
        btCut.setEnabled( active );
        btPaste.setEnabled( active );

        // Last touch, prepare focus, show status and put the divider
        this.spPanelDivision.setVisible( active );

        if ( active ) {
            this.tpMainTab.requestFocusInWindow();
            this.txtSource.requestFocusInWindow();
        }

        return;
    }

    public void viewPage(int pageNumber)
    {
        if ( pageNumber >= 0
          && pageNumber < this.tpMainTab.getTabCount() )
        {
            this.tpMainTab.setSelectedIndex( pageNumber );
        }
    }

    private void showCaretPosition() {
        int lineNumber;
        int columnNumber;
        int currentPos = txtSource.getCaret().getDot();

        lineNumber = txtSource.getLineOfOffset( currentPos );
        columnNumber = currentPos - txtSource.getLineStartOffset( lineNumber );
        ++lineNumber;
        ++columnNumber;

        lblPosition.setText( lineNumber + ":" + columnNumber );
    }

    private void quit() {
        this.onSave(null);
        this.saveConfiguration();
        this.setVisible( false );
    }

    @Override
    public void setTitle(String fileName)
    {
        if ( !fileName.isEmpty() ) {
            super.setTitle( AppInfo.Name + " - " + fileName );
        } else {
            super.setTitle( AppInfo.Name );
        }
    }

    private void setTitle() {
        if ( project == null )
                setTitle( "" );
        else    setTitle( project.getAbsolutePath() );
    }

    private void load(File project) {
        load( project, txtSource );
    }

     private void save(File project) {
        try {
            BufferedWriter output = new BufferedWriter( new FileWriter( project ) );
            output.write( txtSource.getText() );
            output.close();
            txtSource.highlightAllSource();
        } catch(Exception ex) {
            showErrorMsgOnOutput( ex.getMessage() );
        }
    }

    private void prepareOutputForErrors() {
        txtOutput.setText("");
        txtOutput.setForeground(Color.RED);
    }

     private void prepareOutputForInfo() {
        txtOutput.setText("");
        txtOutput.setForeground(Color.BLUE);
    }

    private void load(File project, SourceEditor txt)
    {
        this.setStatus();

        try {
            txt.setFile( project );
        } catch (IOException exc) {
            this.prepareOutputForErrors();
            this.txtOutput.append( exc.getLocalizedMessage() );
            this.setErrorStatus();
        }
    }

    private void lookFor(int initPos, String text) {
        // Set initial position
        if ( initPos > -1 ) {
            this.txtSource.setCaretPosition( initPos );
        } else {
            initPos = this.txtSource.getCaretPosition() +1;
        }

        // Prepare search string
        if ( text == null ) {
            text = "";
        }

        this.searchText = text.trim().toLowerCase();

        // Locate ocurrence
        int pos = this.txtSource.getText().toLowerCase().indexOf( getSearchText(), initPos );
        
        if ( pos > -1 ) {
            this.txtSource.setCaretPosition( pos );
            this.showCaretPosition();
        } else {
            showInfoMsg(L18n.getMsg(Msg.TextNotFound));
        }

        txtSource.requestFocusInWindow();
    }

    /**
     * @return the language
     */
    public Language getLanguage() {
        return core.I18n.language;
    }

    /**
     * @param language the language to set
     */
    public void setLanguage(Language language) {
        core.I18n.language = language;
    }

    public void translateInto(Language language)
    {
        this.setLanguage( language );

        // Menus
        mFile.setText( L18n.getMsg( Msg.MenuFile ) );
        opNewProgram.setText( L18n.getMsg( Msg.Program ) );
        opNewModule.setText( L18n.getMsg( Msg.Module ) );
        mNew.setText( L18n.getMsg( Msg.OptNew ) );
        opOpen.setText( L18n.getMsg( Msg.OptOpen ) );
        opSave.setText( L18n.getMsg( Msg.OptSave ) );
        opClose.setText( L18n.getMsg( Msg.OptClose ) );
        opQuit.setText( L18n.getMsg( Msg.OptQuit ) );

        mEdit.setText( L18n.getMsg( Msg.MenuEdit ) );
        opCopy.setText( L18n.getMsg( Msg.OptCopy ) );
        opCut.setText( L18n.getMsg( Msg.OptCut ) );
        opPaste.setText( L18n.getMsg( Msg.OptPaste ) );
        opFind.setText( L18n.getMsg( Msg.OptFind ) );
        opFindAgain.setText( L18n.getMsg( Msg.OptFindAgain ) );
        opIndentMore.setText( L18n.getMsg( Msg.OptIndentMore ) );
        opIndentLess.setText( L18n.getMsg( Msg.OptIndentLess ) );
        opPreferences.setText( L18n.getMsg( Msg.OptPreferences ) );

        mView.setText( L18n.getMsg( Msg.MenuView ) );
        opViewShortcuts.setText( L18n.getMsg( Msg.OptViewShortcuts ) );
        opViewStatusBar.setText( L18n.getMsg( Msg.OptViewStatusbar ) );
        opViewLineNumbers.setText( L18n.getMsg( Msg.OptViewLineNumbers ) );
        opChangeView.setText( L18n.getMsg( Msg.OptChangeView ) );

        mZoom.setText( L18n.getMsg( Msg.MenuZoom ) );
        opIncZoom.setText( L18n.getMsg( Msg.OptIncZoom ) );
        opDecZoom.setText( L18n.getMsg( Msg.OptIncZoom ) );

        mExecute.setText( L18n.getMsg( Msg.MenuExecute ) );
        opRun.setText( L18n.getMsg( Msg.OptRun ) );
        opDocument.setText( L18n.getMsg( Msg.OptDoc ) );

        mHelp.setText( L18n.getMsg( Msg.MenuHelp ) );
        opHelp.setText( L18n.getMsg( Msg.OptContents ) );
        opAbout.setText( L18n.getMsg( Msg.OptAbout ) );

        // Buttons
        btNew.setToolTipText( L18n.getMsg( Msg.OptNew ) );
        btOpen.setToolTipText( L18n.getMsg( Msg.OptOpen ) );
        btSave.setToolTipText( L18n.getMsg( Msg.OptSave ) );
        btCopy.setToolTipText( L18n.getMsg( Msg.OptCopy ) );
        btCut.setToolTipText( L18n.getMsg( Msg.OptCut ) );
        btPaste.setToolTipText( L18n.getMsg( Msg.OptPaste ) );
        btRun.setToolTipText( L18n.getMsg( Msg.OptRun ) );
    }

    public void translateIntoCurrentLanguage()
    {
        this.translateInto( this.getLanguage() );
    }

    private void setToolbarInitialVisibility(boolean value)
    {
        this.tbTools.setVisible( value );
        this.opViewShortcuts.setSelected( value );
    }

    private void setInitialSize(Dimension size)
    {
        if ( size.width >= this.getMinimumSize().width
          && size.width >= this.getMinimumSize().height )
        {
            this.setSize( size );
        }
    }

    private void saveConfiguration()
    {
        String homeDir = System.getProperty( "user.home" );
        File cnf = new File( homeDir + File.separator + ConfigurationFile );
        FileWriter outStream;

        try {
            outStream = new FileWriter( cnf );
            BufferedWriter out = new BufferedWriter( outStream );
            out.write( "# " + AppInfo.Name + ' ' + AppInfo.Version + '\n' );
            out.write( EtqLanguage + "=" + this.getLanguage().toString() + '\n' );
            out.write( EtqWidth + "=" + Integer.toString( this.getSize().width ) + '\n' );
            out.write( EtqHeight + "=" + Integer.toString( this.getSize().height ) + '\n' );
            out.write( EtqToolbar + "=" + this.tbTools.isVisible() + '\n' );
            out.write( EtqMode + "=" + this.mode.toString() + '\n' );
            out.write( EtqTarget + "=" + this.target.toString() + '\n' );
            out.close();
            outStream.close();
        } catch (IOException ex) {
            showErrorMsg( "Unable to write configuration file:\n" + ex.getMessage() );
        }
    }

    private void loadConfiguration()
    {
        String homeDir = System.getProperty( "user.home" );
        Dimension size = new Dimension( this.getSize() );
        boolean toolbarVisible = true;

        if ( homeDir != null
          && !( homeDir.trim().isEmpty() ) )
        {
            File cnf = new File( homeDir + File.separator + ConfigurationFile );

            if ( cnf.exists() ) {
                FileInputStream inputFile;
                try {
                    inputFile = new FileInputStream( cnf );
                    FileReader fin = new FileReader( cnf );
                    BufferedReader f = new BufferedReader( fin );

                    String line = f.readLine();
                    while( line != null ) {
                        line = line.trim();

                        if ( line.charAt( 0 ) != '#' ) {
                            if ( line.toUpperCase().startsWith( EtqLanguage ) ) {
                                String[] parts = line.split( "=" );
                                parts[ 1 ] = parts[ 1 ].trim().toUpperCase();

                                this.setLanguage( Language.valueOf( parts[ 1 ] ) );
                            }
                            else
                            if ( line.toUpperCase().startsWith( EtqHeight ) ) {
                                String[] parts = line.split( "=" );
                                parts[ 1 ] = parts[ 1 ].trim();

                                size.height = Integer.parseInt( parts[ 1 ] );
                            }
                            else
                            if ( line.toUpperCase().startsWith( EtqWidth ) ) {
                                String[] parts = line.split( "=" );
                                parts[ 1 ] = parts[ 1 ].trim();

                                size.width = Integer.parseInt( parts[ 1 ] );
                            }
                            else
                            if ( line.toUpperCase().startsWith( EtqToolbar ) ) {
                                String[] parts = line.split( "=" );
                                parts[ 1 ] = parts[ 1 ].trim();

                                toolbarVisible = ( parts[ 1 ].equalsIgnoreCase( "TRUE" ) );
                            }
                            else
                            if ( line.toUpperCase().startsWith( EtqMode ) ) {
                                String[] parts = line.split( "=" );
                                parts[ 1 ] = parts[ 1 ].trim();

                                this.mode = Parser.Mode.valueOf( parts[ 1 ] );
                            }
                            else
                            if ( line.toUpperCase().startsWith( EtqTarget ) ) {
                                String[] parts = line.split( "=" );
                                parts[ 1 ] = parts[ 1 ].trim();

                                this.target = Compiler.OutputPlatform.parse(parts[1]);
                            }
                        }

                        line = f.readLine();
                    }

                    // Clean
                    f.close();
                    inputFile.close();

                    // Apply configuration
                    this.translateIntoCurrentLanguage();
                    this.setToolbarInitialVisibility( toolbarVisible );
                    this.setInitialSize( size );
                } catch (Exception ex) {
                    showErrorMsg( "Error reading configuration file:\n" + ex.getMessage() );
                }
            }
        }

        return;
    }

    public void showErrorMsgOnOutput(String msg)
    {
        this.prepareOutputForErrors();
        this.txtOutput.setText( AppInfo.Error + ": " + msg );
    }

    public void showErrorMsg(String msg)
    {
        SwingUtil.showErrorMsg( this, AppInfo.Name, msg );
    }

    public void showInfoMsg(String msg)
    {
        SwingUtil.showInfoMsg( this, AppInfo.Name, msg );
    }

    private File project = null;
    protected String searchText;
    private File libDir;
    private File docDir;
    private Parser.Mode mode = Parser.Mode.Beginner;
    private Compiler.OutputPlatform target = Compiler.OutputPlatform.Js;

    private JButton btCopy;
    private JButton btCut;
    private JButton btFind;
    private JButton btNew;
    private JButton btOpen;
    private JButton btPaste;
    private JButton btRun;
    private JButton btSave;
    private JTextField edSearchText;
    private JLabel lblPosition;
    private JLabel lblStatus;
    private JMenu mEdit;
    private JMenu mExecute;
    private JMenu mFile;
    private JMenu mHelp;
    private JMenuBar mMainMenu;
    private JMenu mNew;
    private JMenu mView;
    private JMenu mZoom;
    private JMenuItem opAbout;
    private JMenuItem opChangeView;
    private JMenuItem opClose;
    private JMenuItem opCopy;
    private JMenuItem opCut;
    private JMenuItem opDecZoom;
    private JMenuItem opDocument;
    private JMenuItem opFind;
    private JMenuItem opFindAgain;
    private JMenuItem opHelp;
    private JMenuItem opIncZoom;
    private JMenuItem opIndentLess;
    private JMenuItem opIndentMore;
    private JMenuItem opNewModule;
    private JMenuItem opNewProgram;
    private JMenuItem opOpen;
    private JMenuItem opPaste;
    private JMenuItem opPreferences;
    private JMenuItem opQuit;
    private JMenuItem opRun;
    private JMenuItem opSave;
    private JCheckBoxMenuItem opViewLineNumbers;
    private JCheckBoxMenuItem opViewShortcuts;
    private JCheckBoxMenuItem opViewStatusBar;
    private JSplitPane spPanelDivision;
    private JScrollPane spOutputScroller;
    private JToolBar tbSearch;
    private JToolBar tbStatus;
    private JToolBar tbTools;
    private JTabbedPane tpBrowse;
    private JTabbedPane tpMainTab;
    private JTextArea txtOutput;
    private SourceEditor txtSource;
    private TextLineNumber textLineNumber;
    private JScrollPane spEditorScroller;
    private DocBrowser docBrowser;
}
